// Any Device - Utilities - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/win32/gsc_os_util_sem.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

#include "gsc_os_util.h"



/*****************************************************************************
*
*	Function:	gsc_os_sem_create
*
*	Purpose:
*
*		Create a semaphore.
*
*	Arguments:
*
*		sem		The structure in which to place the semaphore.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gsc_os_sem_create(gsc_os_sem_t* sem)
{
	int	errs;

	sem->handle	= CreateSemaphore(NULL, 1, 1, NULL);
	errs		= sem->handle ? 0 : 1;
	return(errs);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_create_qty
*
*	Purpose:
*
*		Create a semaphore.
*
*	Arguments:
*
*		sem		The structure in which to place the semaphore.
*
*		cap		The number of units the esmaphore can hold (i.e. capacity).
*
*		put		The number of units to initially put in.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gsc_os_sem_create_qty(gsc_os_sem_t* sem, int cap, int put)
{
	int	errs;

	sem->handle	= CreateSemaphore(NULL, put, cap, NULL);
	errs		= sem->handle ? 0 : 1;
	return(errs);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_destroy
*
*	Purpose:
*
*		Destroy a semaphore.
*
*	Arguments:
*
*		sem		The structure whose content we're to destroy.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gsc_os_sem_destroy(gsc_os_sem_t* sem)
{
	if (sem->handle)
	{
		CloseHandle(sem->handle);
		memset(sem, 0, sizeof(gsc_os_sem_t));
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_get
*
*	Purpose:
*
*		Acquire a semaphore. Do not return until we have the semaphore or
*		there is an error.
*
*	Arguments:
*
*		sem		The structure to access.
*
*	Returned:
*
*		GSC_OS_SEM_GET_OK	We got the semaphore.
*		GSC_OS_SEM_GET_ER	There was an error.
*
*****************************************************************************/

gsc_os_sem_get_t gsc_os_sem_get(gsc_os_sem_t* sem)
{
	gsc_os_sem_get_t	sg;

	sg	= gsc_os_sem_get_to(sem, -1);
	return(sg);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_get_to
*
*	Purpose:
*
*		Acquire a semaphore. Return the timeout value if we don't get the
*		semaphore in the time requested.
*
*	Arguments:
*
*		sem		The structure to access.
*
*	Returned:
*
*		SEM_GET_OK	We got the semaphore.
*		SEM_GET_TO	We timed out.
*		SEM_GET_ER	There was an error.
*
*****************************************************************************/

gsc_os_sem_get_t gsc_os_sem_get_to(gsc_os_sem_t* sem, int timeout_ms)
{
	DWORD				dw;
	gsc_os_sem_get_t	sg;
	DWORD				to	= (timeout_ms < 0) ? INFINITE : timeout_ms;

	if (sem->handle)
	{
		timeBeginPeriod(1);
		dw	= WaitForSingleObject(sem->handle, to);
		timeEndPeriod(1);

		if (dw == WAIT_OBJECT_0)
			sg	= GSC_OS_SEM_GET_OK;
		else if (dw == WAIT_TIMEOUT)
			sg	= GSC_OS_SEM_GET_TO;
		else
			sg	= GSC_OS_SEM_GET_ER;
	}
	else
	{
		sg	= GSC_OS_SEM_GET_ER;
	}

	return(sg);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_put
*
*	Purpose:
*
*		Release a semaphore.
*
*	Arguments:
*
*		sem		The structure to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gsc_os_sem_put(gsc_os_sem_t* sem)
{
	ReleaseSemaphore(sem->handle, 1, NULL);
	return(0);
}



/*****************************************************************************
*
*	Function:	gsc_os_sem_put_qty
*
*	Purpose:
*
*		Release a semaphore.
*
*	Arguments:
*
*		sem		The structure to access.
*
*		int		The number of units to put.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int gsc_os_sem_put_qty(gsc_os_sem_t* sem, int put)
{
	ReleaseSemaphore(sem->handle, put, NULL);
	return(0);
}


