// Any Device - Utilities - Win64 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/common/utils/win64/gsc_os_util_ini.c $
// $Rev: 31425 $
// $Date: 2014-11-13 09:23:43 -0600 (Thu, 13 Nov 2014) $

#include <windows.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "gsc_os_util.h"



/* #defines	*****************************************************************/

#define	SECTION	"defaults"

#if (_MSC_VER >= 1400)  // Visual Studio 2005 or later
	#define	stricmp	_stricmp
#endif



/* variables	*************************************************************/

static	char	_ini_name[MAX_PATH * 2];



/*****************************************************************************
*
*	Function:	gsc_os_ini_init
*
*	Purpose:
*
*		Initialize the .ini code.
*
*	Arguments:
*
*		base_name	The base name of the .ini file.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_os_ini_init(const char* app_name)
{
	GetCurrentDirectory(MAX_PATH, _ini_name);
	strcat(_ini_name, "\\");
	strcat(_ini_name, app_name);
	strcat(_ini_name, ".ini");
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_bool_get
*
*	Purpose:
*
*		Read a boolean from a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign is
*					recorded here.
*
*		defalt		The value to use if the .ini entry is absent or empty.
*
*	Returned:
*
*		0			The default value was used.
*		> 0			The defalt value was not used.
*
*****************************************************************************/

int gsc_os_ini_bool_get(
	const char*	file,
	const char*	section,
	const char*	key,
	int*		value,
	int			defalt)
{
	char	            buf[16];
	int		            i;
	int		            len;
	long unsigned int   ul;

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	len	= (int) GetPrivateProfileString(section, key, "", buf, sizeof(buf), file);

	if (len == 0)
	{
		len			= 0;
		value[0]	= defalt;
	}
	else if (stricmp(buf, "0") == 0)
	{
		value[0]	= FALSE;
	}
	else if (stricmp(buf, "1") == 0)
	{
		value[0]	= TRUE;
	}
	else if (	(stricmp(buf, "0x0") == 0) ||
				(stricmp(buf, "false") == 0) ||
				(stricmp(buf, "no") == 0) ||
				(strchr("FfNn", buf[0])))
	{
		value[0]	= FALSE;
	}
	else if (	(stricmp(buf, "0x1") == 0) ||
				(stricmp(buf, "true") == 0) ||
				(stricmp(buf, "yes") == 0) ||
				(strchr("TtYy", buf[0])))
	{
		value[0]	= TRUE;
	}
	else if ((buf[0] == '0') && (buf[1] == 'x') && (isxdigit(buf[2])))
	{
		sscanf(buf, "0x%lX", &ul);
		value[0]	= ul ? TRUE : FALSE;
	}
	else if (isdigit(buf[0]))
	{
		sscanf(buf, "%d", &i);
		value[0]	= i ? TRUE : FALSE;
	}
	else
	{
		len			= 0;
		value[0]	= defalt;
	}

	return(len);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_bool_set
*
*	Purpose:
*
*		Save a boolean in a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_os_ini_bool_set(
	const char*	file,
	const char*	section,
	const char*	key,
	BOOL		value)
{
	const char*	str;

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	str	= value ? "1" : "0";
	WritePrivateProfileString(section, key, str, file);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_int_get
*
*	Purpose:
*
*		Read an integer from a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign is
*					recorded here.
*
*		defalt		The value to use if the .ini entry is absent or empty.
*
*	Returned:
*
*		0			The default value was used.
*		> 0			The defalt value was not used.
*
*****************************************************************************/

int gsc_os_ini_int_get(
	const char*	file,
	const char*	section,
	const char*	key,
	int*		value,
	int			defalt)
{
	char	buf[16];
	int		len;

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	len			= (int) GetPrivateProfileString(section, key, "", buf, sizeof(buf), file);
	value[0]	= (int) GetPrivateProfileInt(section, key, defalt, file);
	return(len);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_int_set
*
*	Purpose:
*
*		Save an integer in a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_os_ini_int_set(
	const char*	file,
	const char*	section,
	const char*	key,
	int			value)
{
	char	buf[16];

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	sprintf(buf, "%d", value);
	WritePrivateProfileString(section, key, buf, file);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_str_get
*
*	Purpose:
*
*		Read a string from a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The text that appears at the right of the equal sign is
*					recorded here.
*
*		size		The size of the above value string.
*
*	Returned:
*
*		>= 0		The size of the string returned.
*
*****************************************************************************/

int gsc_os_ini_str_get(
	const char*	file,
	const char*	section,
	const char*	key,
	char*		value,
	size_t		size)
{
	int	len;

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	len	= (int) GetPrivateProfileString(section, key, "", value, (DWORD) size, file);
	return(len);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_str_set
*
*	Purpose:
*
*		Save a string in a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The text that appears at the right of the equal sign.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_os_ini_str_set(
	const char*	file,
	const char*	section,
	const char*	key,
	const char*	value)
{
	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	WritePrivateProfileString(section, key, value, file);
}



/*****************************************************************************
*
*	Function:	ini_u32_get
*
*	Purpose:
*
*		Read a U32 from a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign is
*					recorded here.
*
*		defalt		The value to use if the .ini entry is absent or empty.
*
*	Returned:
*
*		0			The default value was used.
*		> 0			The defalt value was not used.
*
*****************************************************************************/

int ini_u32_get(
	const char*	file,
	const char*	section,
	const char*	key,
	U32*		value,
	U32			defalt)
{
	int		            len;
	char	            str[128];
	long unsigned int	ul;

	len	= gsc_os_ini_str_get(file, section, key, str, sizeof(str));

	if ((str[0] == '0') && (str[1] == 'x') && (isxdigit(str[2])))
	{
		sscanf(str, "0x%lX", &ul);
		value[0]	= (U32) ul;
	}
	else
	{
		len			= 0;
		value[0]	= defalt;
	}

	return(len);
}



/*****************************************************************************
*
*	Function:	gsc_os_ini_u32_set
*
*	Purpose:
*
*		Save a U32 in a .ini file.
*
*	Arguments:
*
*		file		The name of the file. If this is NULL then the default is
*					used.
*
*		section		The text that goes in the square brackets.
*
*		key			The text that appears at the left of the equal sign.
*
*		value		The value that appears at the right of the equal sign.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gsc_os_ini_u32_set(
	const char*	file,
	const char*	section,
	const char*	key,
	U32			value)
{
	char	buf[16];

	if (file == NULL)
		file	= _ini_name;

	if (section == NULL)
		section	= SECTION;

	sprintf(buf, "0x%lX", (long) value);
	WritePrivateProfileString(section, key, buf, file);
}


