// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_buf_qty.c $
// $Rev: 31044 $
// $Date: 2014-10-27 17:44:10 -0500 (Mon, 27 Oct 2014) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY		"Buffer Quantity"



/* variables	*************************************************************/

static	int		_max;
static	int		_min;
static	HWND	_qty;
static	HWND	_qty_dec;
static	HWND	_qty_inc;
static	HWND	_qty_range;
static	int		_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	_max	= gui.api_buf ? 2 : 32;
	_min	= 1;

	if (value < _min)
		value	= _min;
	else if (value > _max)
		value	= _max;

	_setting	= value;
	gui.buf_qty	= _setting;
	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	buffer_qty_init
*
*	Purpose:
*
*		Initialize things for the Buffer Quantity option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_qty_init(void)
{
	HDC			dc;
	HFONT		font;
	int			h;
	int			i;
	LOGFONT		lf;
	TEXTMETRIC	tm;

	_qty_dec		= GetDlgItem(gui.dlg, IDC_QTY_DEC);
	_qty_inc		= GetDlgItem(gui.dlg, IDC_QTY_INC);
	_qty_range		= GetDlgItem(gui.dlg, IDC_QTY_RANGE);
	_qty			= GetDlgItem(gui.dlg, IDC_QTY);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, 2);
	_update_and_save(i);

	// Update the font for the buttons.
	dc	= GetDC(_qty_inc);
	h	= GetDeviceCaps(dc, LOGPIXELSY);
	GetTextMetrics(dc, &tm);
	ReleaseDC(_qty_inc, dc);

	memset(&lf, 0, sizeof(lf));
	lf.lfHeight			= -tm.tmHeight;
	lf.lfWidth			= 0;
	lf.lfEscapement		= 0;
	lf.lfOrientation	= 0;
	lf.lfWeight			= tm.tmWeight;
	lf.lfItalic			= 0;
	lf.lfUnderline		= 0;
	lf.lfStrikeOut		= 0;
	lf.lfCharSet		= tm.tmCharSet;
	lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= tm.tmPitchAndFamily;
	lf.lfFaceName[0]	= 0;

	lf.lfHeight			= -MulDiv(14, h, 72);
	lf.lfWeight			= FW_HEAVY ;
	font				= CreateFontIndirect(&lf);
	SendMessage(_qty_inc, WM_SETFONT, (WPARAM) font, 0);
	SendMessage(_qty_dec, WM_SETFONT, (WPARAM) font, 0);
}



/*****************************************************************************
*
*	Function:	buffer_qty_refresh
*
*	Purpose:
*
*		Refresh the Buffer Quantity controls.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_qty_refresh(void)
{
	char	buf[16];
	int		dec;
	int		inc;

	_update_and_save(_setting);
	sprintf(buf, "%d", _setting);
	SetWindowText(_qty, buf);
	SendMessage(_qty, WM_PAINT, 0, 0);

	inc	= (_setting < _max) ? !gui.working : FALSE;
	dec	= (_setting > _min) ? !gui.working : FALSE;
	EnableWindow(_qty_inc, inc);
	SendMessage(_qty_inc, WM_PAINT, 0, 0);
	EnableWindow(_qty_dec, dec);
	SendMessage(_qty_dec, WM_PAINT, 0, 0);

	sprintf(buf, "(range: %d - %d)", _min, _max);
	SetWindowText(_qty_range, buf);
	SendMessage(_qty_range, WM_PAINT, 0, 0);
}



/*****************************************************************************
*
*	Function:	buffer_qty_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Buffer Quantity controls.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void buffer_qty_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_QTY_DEC:

			id	= _setting - 1;
			break;

		case IDC_QTY_INC:

			id	= _setting + 1;
			break;
	}

	_update_and_save(id);
	buffer_qty_refresh();
}


