// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_menu.c $
// $Rev: 31044 $
// $Date: 2014-10-27 17:44:10 -0500 (Mon, 27 Oct 2014) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"



/*****************************************************************************
*
*	Function:	gui_menu_add_int
*
*	Purpose:
*
*		Modify the text of the specified menu to include the given integer
*		value.
*
*	Arguments:
*
*		menu	The menu to modify.
*
*		value	The value to display.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gui_menu_add_int(HMENU menu, int value)
{
	char	buf[16];

	sprintf(buf, "%d", value);
	gui_menu_add_text(menu, buf);
}



/*****************************************************************************
*
*	Function:	gui_menu_add_item_text
*
*	Purpose:
*
*		Modify the text of the specified menu to include the text of the
*		given menu item.
*
*	Arguments:
*
*		menu	The menu to modify.
*
*		id		The menu item whose text we're to add.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the item.
*
*****************************************************************************/

void gui_menu_add_item_text(HMENU menu, int id)
{
	char	item[1024];
	char	parent[1024];
	char*	psz;
	char	tmp[2];

	gui_menu_get_text(menu, parent, sizeof(parent));
	gui_menu_item_get_text(menu, id, item, sizeof(item));
	psz	= strchr(parent, '\t');

	if (psz)
		psz[0]	= 0;

	if (item[0])
	{
		strcat(parent, "\t(");
		tmp[1]	= 0;

		for (psz = item; psz[0]; psz++)
		{
			if (psz[0] != '&')
			{
				tmp[0]	= psz[0];
				strcat(parent, tmp);
			}
		}

		strcat(parent, ")");
	}

	gui_menu_set_text(menu, parent);
}



/*****************************************************************************
*
*	Function:	gui_menu_add_text
*
*	Purpose:
*
*		Modify the text of the specified menu to include the given text.
*
*	Arguments:
*
*		menu	The menu to modify.
*
*		text	The text we're to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gui_menu_add_text(HMENU menu, const char* text)
{
	char	parent[1024];
	char*	psz;

	gui_menu_get_text(menu, parent, sizeof(parent));
	psz	= strchr(parent, '\t');

	if (psz)
		psz[0]	= 0;

	if ((text) && (text[0]))
	{
		strcat(parent, "\t(");
		strcat(parent, text);
		strcat(parent, ")");
	}

	gui_menu_set_text(menu, parent);
}



/*****************************************************************************
*
*	Function:	gui_menu_get_text
*
*	Purpose:
*
*		Retrieve the text of the given menu and put it in the given buffer.
*
*	Arguments:
*
*		menu	The menu containing the specified item.
*
*		buf		Put the text here.
*
*		size	The size of the above buffer.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the item.
*
*****************************************************************************/

void gui_menu_get_text(HMENU menu, char* buf, size_t size)
{
	int				count;
	int				i;
	MENUITEMINFO	info;
	int				j;
	HMENU			parent;

	buf[0]	= 0;
	parent	= gui_menu_locate_parent_by_sub(gui.menu, menu);
	count	= GetMenuItemCount(parent);
	memset(&info, 0, sizeof(info));
	info.cbSize	= sizeof(info);
	info.fMask	= MIIM_SUBMENU;

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemInfo(parent, i, TRUE, &info);

		if ((j) && (info.hSubMenu == menu))
		{
			GetMenuString(parent, i, buf, (int) size, MF_BYPOSITION);
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	gui_menu_item_add_int
*
*	Purpose:
*
*		Modify the text of the specified menu item to include the text of the
*		given integer value.
*
*	Arguments:
*
*		menu	The menu containing the specified item.
*
*		id		The menu item whose text we want.
*
*		value	The value to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gui_menu_item_add_int(HMENU menu, int id, int value)
{
	char	buf[16];

	sprintf(buf, "%d", value);
	gui_menu_item_add_text(menu, id, buf);
}



/*****************************************************************************
*
*	Function:	gui_menu_item_add_text
*
*	Purpose:
*
*		Modify the text of the specified menu item to include the given text.
*
*	Arguments:
*
*		menu	The menu containing the specified item.
*
*		id		The menu item whose text we want.
*
*		text	The text to add.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void gui_menu_item_add_text(HMENU menu, int id, const char* text)
{
	char	item[1024];
	char*	psz;

	gui_menu_item_get_text(menu, id, item, sizeof(item));
	psz	= strchr(item, '\t');

	if (psz)
		psz[0]	= 0;

	if ((text) && (text[0]))
	{
		strcat(item, "\t(");
		strcat(item, text);
		strcat(item, ")");
	}

	ModifyMenu(	menu,
				id,
				MF_BYCOMMAND,
				(UINT_PTR) id,
				item);
	DrawMenuBar(gui.dlg);
}



/*****************************************************************************
*
*	Function:	gui_menu_item_get_text
*
*	Purpose:
*
*		Retrieve the text of the given menu item and put it in the given
*		buffer.
*
*	Arguments:
*
*		menu	The menu containing the specified item.
*
*		id		The menu item whose text we want.
*
*		buf		Put the text here.
*
*		size	The size of the above buffer.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the item.
*
*****************************************************************************/

void gui_menu_item_get_text(HMENU menu, int id, char* buf, size_t size)
{
	GetMenuString(menu, id, buf, (int) size, MF_BYCOMMAND);
}



/*****************************************************************************
*
*	Function:	gui_menu_locate_parent_by_id
*
*	Purpose:
*
*		Locate the menu containing the given menu item identifier.
*
*	Arguments:
*
*		menu	The menu handle to search.
*
*		id		The menu item to locate.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the item.
*
*****************************************************************************/

HMENU gui_menu_locate_parent_by_id(HMENU menu, int id)
{
	int				count;
	int				i;
	MENUITEMINFO	info;
	int				j;
	HMENU			parent	= NULL;

	count	= GetMenuItemCount(menu);

	// Seach this menu.

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemID(menu, i);

		if (j == id)
		{
			parent	= menu;
			count	= 0;
		}
	}

	// Check submenus.

	memset(&info, 0, sizeof(info));
	info.cbSize	= sizeof(info);
	info.fMask	= MIIM_SUBMENU;

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemInfo(menu, i, TRUE, &info);

		if ((j) && (info.hSubMenu))
		{
			parent	= gui_menu_locate_parent_by_id(info.hSubMenu, id);

			if (parent)
				count	= 0;
		}
	}

	return(parent);
}



/*****************************************************************************
*
*	Function:	gui_menu_locate_parent_by_sub
*
*	Purpose:
*
*		Locate the menu containing the given submenu.
*
*	Arguments:
*
*		menu	The menu handle to search.
*
*		sub		The submenu to locate.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the submenu.
*
*****************************************************************************/

HMENU gui_menu_locate_parent_by_sub(HMENU menu, HMENU submenu)
{
	int				count;
	int				i;
	MENUITEMINFO	info;
	int				j;
	HMENU			parent	= NULL;

	count	= GetMenuItemCount(menu);
	memset(&info, 0, sizeof(info));
	info.cbSize	= sizeof(info);
	info.fMask	= MIIM_SUBMENU;

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemInfo(menu, i, TRUE, &info);

		if ((j) && (info.hSubMenu == submenu))
		{
			parent	= menu;
			count	= 0;
		}
	}

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemInfo(menu, i, TRUE, &info);

		if ((j) && (info.hSubMenu))
		{
			parent	= gui_menu_locate_parent_by_sub(info.hSubMenu, submenu);

			if (parent)
				count	= 0;
		}
	}

	return(parent);
}



/*****************************************************************************
*
*	Function:	gui_menu_set_text
*
*	Purpose:
*
*		Update the text of the given menu.
*
*	Arguments:
*
*		menu	The menu containing the specified item.
*
*		text	The desired menu text.
*
*	Returned:
*
*		NULL	The menu or submenu was not found.
*		else	The handle to the menu or submenu containing the item.
*
*****************************************************************************/

void gui_menu_set_text(HMENU menu, const char* text)
{
	int				count;
	int				i;
	MENUITEMINFO	info;
	int				j;
	HMENU			parent;

	parent	= gui_menu_locate_parent_by_sub(gui.menu, menu);
	count	= GetMenuItemCount(parent);
	memset(&info, 0, sizeof(info));
	info.cbSize	= sizeof(info);
	info.fMask	= MIIM_SUBMENU;

	for (i = 0; i < count; i++)
	{
		j	= GetMenuItemInfo(parent, i, TRUE, &info);

		if ((j) && (info.hSubMenu == menu))
		{
			ModifyMenu(	parent,
						i,
						MF_BYPOSITION,
						(UINT_PTR) menu,
						text);
			DrawMenuBar(gui.dlg);
			break;
		}
	}
}


