// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_status.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "gsc_os_util.h"



/* variables	*************************************************************/

static	HWND	_status;



/*****************************************************************************
*
*	Function:	status_init
*
*	Purpose:
*
*		Initialize things for the Status window.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void status_init(void)
{
	HDC			dc;
	HFONT		font;
	int			h;
	LOGFONT		lf;
	TEXTMETRIC	tm;

	_status	= GetDlgItem(gui.dlg, IDC_STATUS);

	// Update the status list box font.
	dc	= GetDC(_status);
	h	= GetDeviceCaps(dc, LOGPIXELSY);
	GetTextMetrics(dc, &tm);
	ReleaseDC(_status, dc);

	memset(&lf, 0, sizeof(lf));
	lf.lfHeight			= -tm.tmHeight;
	lf.lfWidth			= 0;
	lf.lfEscapement		= 0;
	lf.lfOrientation	= 0;
	lf.lfWeight			= tm.tmWeight;
	lf.lfItalic			= 0;
	lf.lfUnderline		= 0;
	lf.lfStrikeOut		= 0;
	lf.lfCharSet		= tm.tmCharSet;
	lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= tm.tmPitchAndFamily;
	lf.lfFaceName[0]	= 0;

	lf.lfHeight			= -MulDiv(10, h, 72);
	lf.lfPitchAndFamily	= FIXED_PITCH;
	lf.lfWeight			= FW_MEDIUM;
	font				= CreateFontIndirect(&lf);
	SendMessage(_status, WM_SETFONT, (WPARAM) font, 0);
}



/*****************************************************************************
*
*	Function:	status_reset
*
*	Purpose:
*
*		Reset the status window.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void status_reset(void)
{
	SendMessage(_status, LB_RESETCONTENT, 0, 0);
	SendMessage(_status, WM_PAINT, 0, 0);
}



/*****************************************************************************
*
*	Function:	printf
*
*	Purpose:
*
*		Implement a global printf() replacement so output is posted to the
*		dialog.
*
*	Arguments:
*
*		format	The format string to use.
*
*		...		Any additional arguments.
*
*	Returned:
*
*		The value normally returned by printf().
*
*****************************************************************************/

int printf(const char* format, ...)
{
	static	int				char_no;
	static	int				init;
	static	int				line_no;
	static	char			line[4096];
	static	gsc_os_sem_t	sem;

	char	buf[4096];
	LRESULT	i;
	char*	psz;
	int		ret;
	va_list	val;

	if (init == 0)
	{
		init	= 1;
		gsc_os_sem_create(&sem);
	}

	gsc_os_sem_get(&sem);

	va_start(val, format);
	ret	= vsprintf(buf, format, val);
	va_end(val);

	i	= SendMessage(_status, LB_GETCOUNT, 0, 0);

	if (i == 0)
	{
		char_no	= 0;
		line_no	= 0;
		line[0]	= 0;
	}

	for (psz = buf; ; psz++)
	{
		if (psz[0] == 0)
		{
			if (line[0] == 0)
				break;

			// Add this partial line to the list box.
			SendMessage(_status, LB_DELETESTRING, line_no, 0);
			SendMessage(_status, LB_INSERTSTRING, line_no, (LPARAM) line);
			SendMessage(_status, LB_SETCURSEL, line_no, 0);
			SendMessage(_status, WM_PAINT, 0, 0);
			break;
		}

		if (psz[0] == '\n')
		{
			// Add this partial line to the list box.
			SendMessage(_status, LB_DELETESTRING, line_no, 0);
			SendMessage(_status, LB_INSERTSTRING, line_no, (LPARAM) line);
			SendMessage(_status, LB_SETCURSEL, line_no, 0);
			SendMessage(_status, WM_PAINT, 0, 0);
			char_no	= 0;
			line[0]	= 0;
			line_no++;
			continue;
		}

		line[char_no]	= psz[0];
		char_no++;
		line[char_no]	= 0;
	}

	gsc_os_sem_put(&sem);
	return(ret);
}


