// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/ringbuf/win32/gui_tx_io_data_size.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "resource.h"
#include "hpdi32_util.h"
#include "gsc_util.h"



/* #defines	*****************************************************************/

#define	KEY	"HPDI32_IO_DATA_SIZE__TX"



/* variables	*************************************************************/

static	HMENU		_menu;
static	const char*	_name;
static	int			_setting;



/*****************************************************************************
*
*	Function:	_update_and_save
*
*	Purpose:
*
*		Update the menu check state and save the setting.
*
*	Arguments:
*
*		value	The current setting.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _update_and_save(int value)
{
	switch (value)
	{
		default:
		case HPDI32_IO_DATA_SIZE_8_BITS:

			_name		= "HPDI32_IO_DATA_SIZE_8_BITS";
			_setting	= HPDI32_IO_DATA_SIZE_8_BITS;
			gui_menu_add_item_text(_menu, IDC_TX_IO_DATA_SIZE_08);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_08, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_16, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_32, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_IO_DATA_SIZE_16_BITS:

			_name		= "HPDI32_IO_DATA_SIZE_16_BITS";
			_setting	= HPDI32_IO_DATA_SIZE_16_BITS;
			gui_menu_add_item_text(_menu, IDC_TX_IO_DATA_SIZE_16);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_08, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_16, MF_BYCOMMAND | MF_CHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_32, MF_BYCOMMAND | MF_UNCHECKED);
			break;

		case HPDI32_IO_DATA_SIZE_32_BITS:

			_name		= "HPDI32_IO_DATA_SIZE_32_BITS";
			_setting	= HPDI32_IO_DATA_SIZE_32_BITS;
			gui_menu_add_item_text(_menu, IDC_TX_IO_DATA_SIZE_32);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_08, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_16, MF_BYCOMMAND | MF_UNCHECKED);
			CheckMenuItem(_menu, IDC_TX_IO_DATA_SIZE_32, MF_BYCOMMAND | MF_CHECKED);
			break;
	}

	gsc_os_ini_int_set(NULL, NULL, KEY, _setting);
}



/*****************************************************************************
*
*	Function:	tx_io_data_size_init
*
*	Purpose:
*
*		Initialize things for the Tx I/O Data Size option.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_data_size_init(void)
{
	int		i;

	_menu	= GetMenu(gui.dlg);
	_menu	= gui_menu_locate_parent_by_id(_menu, IDC_TX_IO_DATA_SIZE_08);
	gsc_os_ini_int_get(NULL, NULL, KEY, &i, HPDI32_IO_DATA_SIZE_DEFAULT);
	_update_and_save(i);
}



/*****************************************************************************
*
*	Function:	tx_io_data_size_refresh
*
*	Purpose:
*
*		Refresh the Tx I/O Data Size menu items.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_data_size_refresh(void)
{
	if (gui.working)
	{
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_08, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_16, MF_BYCOMMAND | MF_GRAYED);
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_32, MF_BYCOMMAND | MF_GRAYED);
	}
	else
	{
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_08, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_16, MF_BYCOMMAND | MF_ENABLED);
		EnableMenuItem(_menu, IDC_TX_IO_DATA_SIZE_32, MF_BYCOMMAND | MF_ENABLED);
	}
}



/*****************************************************************************
*
*	Function:	tx_io_data_size_wm_command
*
*	Purpose:
*
*		Process a WM_COMMAND message for a Tx I/O Data Size control or menu
*		items.
*
*	Arguments:
*
*		id		The id of the control or menu item selected.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void tx_io_data_size_wm_command(int id)
{
	switch (id)
	{
		default:
		case IDC_TX_IO_DATA_SIZE_08:

			id	= HPDI32_IO_DATA_SIZE_8_BITS;
			break;

		case IDC_TX_IO_DATA_SIZE_16:

			id	= HPDI32_IO_DATA_SIZE_16_BITS;
			break;

		case IDC_TX_IO_DATA_SIZE_32:

			id	= HPDI32_IO_DATA_SIZE_32_BITS;
			break;
	}

	_update_and_save(id);
}



/*****************************************************************************
*
*	Function:	tx_io_data_size_config
*
*	Purpose:
*
*		Configure the given device's Tx I/O Data Size parameter.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

int tx_io_data_size_config(void* dev)
{
	int	errs	= 0;
	U32	status;

	gsc_label_text(KEY);
	status	= HPDI32_IO_DATA_SIZE__TX_SET(dev, _setting);
	errs	= label_fail_func(1, "HPDI32_IO_DATA_SIZE__TX_SET", status);

	if (errs == 0)
		printf("PASS  (%s)\n", _name);

	return(errs);
}


