// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/rx_simple/rx_simple.c $
// $Rev: 39022 $
// $Date: 2017-06-13 17:03:48 -0500 (Tue, 13 Jun 2017) $

#include "main.h"



// variables ******************************************************************

static	U32	buffer[256L * 1024];	// 1MB of data.



/******************************************************************************
*
*	Function:	_config
*
*	Purpose:
*
*		Configure things in preparation for the test.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _config(void* handle)
{
	int	errs	= 0;
	U32	status;

	printf("Configuring:\n");

	printf("  Initialize: ");
	status	= hpdi32_init(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Rx I/O Mode: DMDMA: ");
	status	= HPDI32_IO_MODE__RX_DMDMA(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	// This is done in case we are connected to another HPDI32.
	printf("  Tx/Rx Tri-State Mode: Enable: ");
	status	= HPDI32_MISC_TX_RX_TRI_STATE__YES(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Rx Enable: Yes: ");
	status	= HPDI32_RX_ENABLE__YES(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	return(errs);
}



/******************************************************************************
*
*	Function:	_tx_sim_enable
*
*	Purpose:
*
*		Configure the HPDI32 to simulate presence of another device sending us
*		data. This is done by configuring the flow control signals as GPIO
*		inputs. Doing this disables the flow control functionality. We then
*		enable the transmitter, which enables the Tx Clock. With this
*		configuration the data is received at the rate of the Tx Clock.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _tx_sim_enable(void* handle)
{
	int	errs	= 0;
	U32	status;

	printf("Tx Simulation:\n");

	printf("  Frame Valid: GPIO Input: ");
	status	= HPDI32_CABLE_COMMAND_MODE__FV_IN(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Line Valid: GPIO Input: ");
	status	= HPDI32_CABLE_COMMAND_MODE__LV_IN(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Status Valid: GPIO Input: ");
	status	= HPDI32_CABLE_COMMAND_MODE__SV_IN(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Rx Ready: GPIO Input: ");
	status	= HPDI32_CABLE_COMMAND_MODE__RR_IN(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Tx Ready: GPIO Input: ");
	status	= HPDI32_CABLE_COMMAND_MODE__TR_IN(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	printf("  Tx Enable: Yes: ");
	status	= HPDI32_TX_ENABLE__YES(handle);
	printf("%s\n", (status == GSC_SUCCESS) ? "PASS" : "FAIL <---");
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	return(errs);
}



/******************************************************************************
*
*	Function:	_rx_data
*
*	Purpose:
*
*		Receive the data.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _rx_data(void* handle)
{
	size_t		bytes	= sizeof(buffer);
	int			errs	= 0;
	const char*	factor	= "";
	int			i;
	U32			sent	= 0;
	U32			status;
	long long	total	= 0;

	printf("Receiving Data:\n");

	for (i = 0; i < 1024; i++)
	{
		if ((i % 64) == 0)
			printf("\n  ");

		printf(".");
		sent	= 0;
		status	= hpdi32_read(handle, buffer, sizeof(buffer), &sent);

		if (sent > 0)
			total	+= sent;

		if (status != GSC_SUCCESS)
		{
			printf(" FAIL <---\n");
			errs++;
			break;
		}

		if (sent != sizeof(buffer))
		{
			printf(" FAIL <--- (Read only %ld bytes)", (long) sent);
			errs++;
			break;
		}
	}

	printf("\n\n");

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "K";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "M";
	}

	if ((total ) && ((total % 1024) == 0))
	{
		total	/= 1024;
		factor	= "G";
	}

	printf("Read %lld%s Bytes\n", total, factor);
	return(errs);
}



/******************************************************************************
*
*	Function:	rx_data
*
*	Purpose:
*
*		Perform a simple data receive operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*		tx_sim	Simlate presence of a Tx device providing data.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int	rx_data(void* handle, int tx_sim)
{
	int		errs		= 0;
	int		fifos		= 1;

	memset(buffer, 0, sizeof(buffer));

	errs	+= _config(handle);

	if ((errs == 0) && (tx_sim))
		errs	+= _tx_sim_enable(handle);

	if (errs == 0)
		errs	+= _rx_data(handle);

	return(errs);
}


