// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/about.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement the About box.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "about.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME	"about"



/* typedefs	*****************************************************************/

typedef struct
{
	device_t*	device;
	HWND		dlg;
} data_t;



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	data_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (data_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
		data->dlg	= NULL;

	win_pos_save(dlg, SECTION_NAME);
	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id				flags						*/
		{ -1,				COPY_EOL | COPY_UNDERLINE	},

		{ IDC_COMPANY,		COPY_EOL					},
		{ IDC_APPLICATION,	COPY_EOL					},
		{ IDC_VERSION,		COPY_EOL					},
		{ IDC_COPYRIGHT,	COPY_EOL					},

		{ 0,				COPY_UNDERLINE				},
		{ 0,				COPY_EOL					},

		{ IDCANCEL,			COPY_EOL					}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam)
{
	int		id	= LOWORD(wParam);
	LRESULT	ret	= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the about dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	data_t*	data;
	TCHAR	version[64];

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (data_t*) lParam;

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	set_win_title(dlg, data->device, FALSE, FALSE, "About");

	sprintf(version, "Version %s", VERSION_STRING);
	SetDlgItemText(dlg, IDC_VERSION, version);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the About dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	data_t	data;

	if (data.dlg)	/*lint !e727 */
	{
		SetFocus(data.dlg);
	}
	else
	{
		data.device	= device;
		data.dlg	= CreateDialogParam(device->instance,
										(LPCTSTR) IDD_ABOUT,
										dlg,
										(DLGPROC) _dlg_proc,
										(LPARAM) &data);
		ShowWindow(data.dlg, SW_SHOWNORMAL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	about_service
*
*	Purpose:
*
*		Implement the common service entry point for the About services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT about_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


