// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/cable_cmd.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement common support for the Cable Command buttons.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "cable.h"
#include "copy.h"
#include "gui.h"
#include "resource.h"
#include "util_ctrl.h"



/* variables	*************************************************************/

static const ctrl_id_u32_t	_mode_list[]	=
{
	/* ctrl_id						u32										*/
	{ IDC_CABLE_CMD_FLOW_CONTROL,	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL	},
	{ IDC_CABLE_CMD_GPIO_IN,		HPDI32_CABLE_COMMAND_MODE_GPIO_IN		},
	{ IDC_CABLE_CMD_GPIO_LOW,		HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW	},
	{ IDC_CABLE_CMD_GPIO_HIGH,		HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI	}
};

static const value_list_entry_t	_command_value_list[]	=
{
	{
	/* user_text			*/	"Flow Control",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL),
	/* suffix				*/	"_FC"
	},
	{
	/* user_text			*/	"GPIO Input",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_CABLE_COMMAND_MODE_GPIO_IN),
	/* suffix				*/	"_IN"
	},
	{
	/* user_text			*/	"GPIO Low Output",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW),
	/* suffix				*/	"_LOW"
	},
	{
	/* user_text			*/	"GPIO High Output",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI),
	/* suffix				*/	"_HI"
	}
};

static const value_list_t	_command_values[]	=
{
	{
	/* list	*/	_command_value_list,
	/* size	*/	ARRAY_ELEMENTS(_command_value_list)
	}
};

const value_t	cable_command_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_command_values
	}
};

static	const parm_entry_t*	_cable_parm_list[]	=
{
	&cable_command_0_mode,
	&cable_command_1_mode,
	&cable_command_2_mode,
	&cable_command_3_mode,
	&cable_command_4_mode,
	&cable_command_5_mode,
	&cable_command_6_mode
};

const parm_list_t	cable_parm_list	=
{
	/* list		*/	_cable_parm_list,
	/* size		*/	ARRAY_ELEMENTS(_cable_parm_list),
	/* suffix	*/	NULL,
	/* which	*/	0
};



/*****************************************************************************
*
*	Function:	_common_init
*
*	Purpose:
*
*		Perform some common initialization steps.
*
*	Arguments:
*
*		data		The window specific data we track.
*
*		in_buf		The title of the GPIO Input option goes here.
*
*		low_buf		The title of the GPIO Output Low option goes here.
*
*		high_buf	The title of the GPIO Output High option goes here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _common_init(
	cable_data_t*	data,
	char*			in_buf,
	char*			low_buf,
	char*			high_buf)
{
	sprintf(in_buf, "GPIO %d: Input", data->gpio_index);
	sprintf(low_buf, "GPIO %d: Output Low", data->gpio_index);
	sprintf(high_buf, "GPIO %d: Output High", data->gpio_index);
}



/*****************************************************************************
*
*	Function:	_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_read(cable_data_t* data)
{
	U32	stat;
	U32	status;

	status	= HPDI32_CABLE_COMMAND_MODE__GET(	data->device->handle,
												data->which,
												&data->mode.setting);
	stat	= HPDI32_MISC_FEATURES__GET(data->device->handle,
										data->feature,
										&data->present);
	status	= (status == GSC_SUCCESS) ? stat : status;
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_write(cable_data_t* data)
{
	U32	status;

	status	= HPDI32_CABLE_COMMAND_MODE__SET(	data->device->handle,
												data->which,
												data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mode_dialog_read(HWND dlg, cable_data_t* data)
{
	data->mode.setting	= ctrl_radio_2_u32(	dlg,
											_mode_list,
											ARRAY_ELEMENTS(_mode_list),
											data->mode.defalt);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _mode_dialog_write(HWND dlg, cable_data_t* data)
{
	unsigned int		other;
	unsigned int		primary;
	U32					u32;

	if (data->device->handle == NULL)
	{
		// Disable and uncheck everything.
		primary	= CTRL_DISABLE;
		other	= CTRL_DISABLE;
		u32		= (U32) -1;
	}
	else if (data->present == HPDI32_MISC_FEATURES_ABSENT)
	{
		// GPIO is unavailable.
		primary	= CTRL_ENABLE;
		other	= CTRL_DISABLE;
		u32		= _mode_list[0].u32;
	}
	else
	{
		// All options are available.
		primary	= CTRL_ENABLE;
		other	= CTRL_ENABLE;
		u32		= data->mode.setting;
	}

	ctrl_u32_2_radio(dlg, u32, _mode_list, ARRAY_ELEMENTS(_mode_list));
	ctrl_menu_set(dlg, IDC_CABLE_CMD_FUNCTION, primary);
	ctrl_menu_set(dlg, IDC_CABLE_CMD_FLOW_CONTROL, primary);
	ctrl_menu_set(dlg, IDC_CABLE_CMD_GPIO_IN, other);
	ctrl_menu_set(dlg, IDC_CABLE_CMD_GPIO_LOW, other);
	ctrl_menu_set(dlg, IDC_CABLE_CMD_GPIO_HIGH, other);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(cable_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(cable_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, cable_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, cable_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	cable_data_t*	data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	char			name[64];
	U32				stat;
	U32				status	= GSC_SUCCESS;

	data	= (cable_data_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(name, "Cable");
	strcat(name, data->changes ? " *" : "");
	set_win_title(dlg, device, FALSE, TRUE, name);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			// Update everything from the device.
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		flags			= CTRL_ENABLE;
		data->was_open	= TRUE;
	}
	else
	{
		// Everything is essentially disabled.
		flags			= CTRL_DISABLE;
		data->changes	= FALSE;
		data->was_open	= FALSE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, cable_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Respond to an DEFAULT request.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, cable_data_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->mode.setting	= data->mode.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	cable_data_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (cable_data_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		data->dlg	= NULL;
		win_pos_save(dlg, data->section_name);
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, cable_data_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_idc_cable_cmd_mode
*
*	Purpose:
*
*		Respond to activity on the MODE radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		TRUE	We processed the message.
*		FALSE	We did not process the message.
*
*****************************************************************************/

static LRESULT _idc_cable_cmd_mode(
	HWND			dlg,
	WPARAM			wParam,
	LPARAM			lParam,
	cable_data_t*	data)
{
	int		code;
	HWND	control;
	LRESULT	ret;
	LRESULT	state;

	if (data->ignore)
	{
		ret	= TRUE;
	}
	else
	{
		code	= HIWORD(wParam);
		control	= (HWND) lParam;

		if (code == BN_SETFOCUS)
		{
			state	= SendMessage(control, BM_GETCHECK, 0, 0);
			ret		= FALSE;

			if (state == BST_UNCHECKED)
				data->mode.control	= control;
			else
				data->mode.control	= NULL;
		}
		else if ((code == BN_CLICKED) &&
				(data->mode.control == control))
		{
			// We get here only if a change has occurred.
			data->changes	= TRUE;
			_refresh_window(dlg, data->device, NULL);
			ret	= TRUE;
		}
		else
		{
			ret	= FALSE;
		}
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id							flags						*/
		{ -1,							COPY_EOL | COPY_UNDERLINE	},

		#define	_FLAGS_		(COPY_INDENT | COPY_EOL | COPY_DISABLED \
							| COPY_BM_GETCHECK | COPY_WIDTH | 19)

		{ IDC_CABLE_CMD_FUNCTION,		COPY_EOL					},
		{ IDC_CABLE_CMD_FLOW_CONTROL,	_FLAGS_						},
		{ IDC_CABLE_CMD_GPIO_IN,		_FLAGS_						},
		{ IDC_CABLE_CMD_GPIO_LOW,		_FLAGS_						},
		{ IDC_CABLE_CMD_GPIO_HIGH,		_FLAGS_						},

		{ 0,			COPY_UNDERLINE								},
		{ 0,			COPY_EOL									},

		{ IDOK,			COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDCANCEL,		COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_APPLY,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	},
		{ IDC_DEFAULT,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8	}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	cable_data_t*	data	= (cable_data_t*) GET_WINDOW_USER_DATA(dlg);
	int				id		= LOWORD(wParam);
	LRESULT			ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_CABLE_CMD_FLOW_CONTROL:
		case IDC_CABLE_CMD_GPIO_IN:
		case IDC_CABLE_CMD_GPIO_LOW:
		case IDC_CABLE_CMD_GPIO_HIGH:

			ret	= _idc_cable_cmd_mode(dlg, wParam, lParam, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the cable items.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char			buf[128];
	char			buf_in[128];
	char			buf_high[128];
	char			buf_low[128];
	cable_data_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (cable_data_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	sprintf(buf, "Command %d", data->cmd_index);
	ctrl_menu_set_text(dlg, IDC_CABLE_CMD_FUNCTION, buf);

	_common_init(data, buf_in, buf_low, buf_high);

	ctrl_menu_set_text(dlg, IDC_CABLE_CMD_FLOW_CONTROL, data->title);
	ctrl_menu_set_text(dlg, IDC_CABLE_CMD_GPIO_IN, buf_in);
	ctrl_menu_set_text(dlg, IDC_CABLE_CMD_GPIO_LOW, buf_low);
	ctrl_menu_set_text(dlg, IDC_CABLE_CMD_GPIO_HIGH, buf_high);

	win_pos_restore(dlg, data->section_name, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the Open dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}




/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	cable_data_t*	data	= (cable_data_t*) wParam;

	if (data->dlg)
	{
		SetFocus(data->dlg);
	}
	else
	{
		data->changes	= FALSE;
		data->ignore	= TRUE;
		data->was_open	= FALSE;
		data->dlg		= CreateDialogParam(data->device->instance,
											(LPCTSTR) IDD_CABLE_CMD,
											dlg,
											(DLGPROC) _dlg_proc,
											(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
		data->ignore	= FALSE;
		_refresh_window(data->dlg, data->device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	cable_command_service
*
*	Purpose:
*
*		Implement the common service entry point for the Cable Command X
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT cable_command_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	cable_command_cfg_wiz
*
*	Purpose:
*
*		Provide customized processing for the Configuration Wizard's processing
*		of the Cable Command configuration options.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		entry		The structure for the parameter to process.
*
*		data		The cable data we're to use.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

U32 cable_command_cfg_wiz(HWND dlg, const parm_entry_t* entry, cable_data_t* data)
{
	char		buf_in[128];
	char		buf_high[128];
	char		buf_low[128];
	const char*	msg	= " (This feature is unavailable.)";
	U32			status;

	_common_init(data, buf_in, buf_low, buf_high);
	status	= _mode_device_read(data);

	if (data->present == HPDI32_MISC_FEATURES_ABSENT)
	{
		// GPIO is unavailable.
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_2, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_3, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_CFG_WIZ_DATA_RADIO_4, CTRL_DISABLE);
		strcat(buf_in, msg);
		strcat(buf_low, msg);
		strcat(buf_high, msg);
	}

	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_RADIO_1, data->title);
	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_RADIO_2, buf_in);
	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_RADIO_3, buf_low);
	ctrl_menu_set_text(dlg, IDC_CFG_WIZ_DATA_RADIO_4, buf_high);
	return(status);
}


