// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gui.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement the GUI window.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>

#include "about.h"
#include "align.h"
#include "arrow.h"
#include "button.h"
#include "c_code.h"
#include "cable.h"
#include "close.h"
#include "config.h"
#include "device_id.h"
#include "gpio.h"
#include "gui.h"
#include "init.h"
#include "misc.h"
#include "open.h"
#include "preference.h"
#include "regs.h"
#include "resource.h"
#include "rx.h"
#include "select.h"
#include "tx.h"
#include "util_ctrl.h"
#include "version.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME	"gui"



/* typedefs *****************************************************************/

typedef struct
{
	device_t*	device;
} gui_t;



/* variables *****************************************************************/

static const service_item_t	_service_list[]	=
{
	/* function						*/
	{ select_service				},
	{ open_service					},
	{ close_service					},
	{ init_service					},
	{ config_wizard_service			},
	{ config_save_service			},
	{ config_restore_service		},
	{ device_id_service				},
	{ regs_fw_service				},
	{ regs_pci_service				},
	{ regs_plx_service				},
	{ c_code_service				},

	{ tx_enable_service				},
	{ tx_disable_service			},
	{ tx_enabled_service			},
	{ tx_oscillator_service			},
	{ tx_clock_divider_service		},
	{ tx_clock_service				},
	{ tx_transmit_service			},
	{ tx_fifo_service				},
	{ tx_data_service				},
	{ tx_io_service					},
	{ tx_rx_ready_service			},
	{ tx_data_ready_service			},
	{ tx_frame_valid_service		},
	{ tx_line_valid_service			},
	{ tx_status_valid_service		},
	{ tx_service					},

	{ rx_enable_service				},
	{ rx_disable_service			},
	{ rx_enabled_service			},
	{ rx_clock_service				},
	{ rx_receive_service			},
	{ rx_fifo_service				},
	{ rx_data_service				},
	{ rx_io_service					},
	{ rx_rx_ready_service			},
	{ rx_frame_valid_service		},
	{ rx_line_valid_service			},
	{ rx_status_valid_service		},
	{ rx_service					},

	{ gpio_0_service				},
	{ gpio_1_service				},
	{ gpio_2_service				},
	{ gpio_3_service				},
	{ gpio_4_service				},
	{ gpio_5_service				},
	{ gpio_6_service				},
	{ gpio_read_service				},
	{ gpio_write_service			},
	{ gpio_service					},

	{ cable_clock_service			},
	{ cable_data_service			},
	{ cable_command_0_service		},
	{ cable_command_1_service		},
	{ cable_command_2_service		},
	{ cable_command_3_service		},
	{ cable_command_4_service		},
	{ cable_command_5_service		},
	{ cable_command_6_service		},
	{ cable_service					},

	{ misc_favor_tx_service			},
	{ misc_tx_rx_tri_state_service	},

	{ version_data_service			},

	{ NULL							}
};



/*****************************************************************************
*
*	Function:	_get_dialog_alignment_data
*
*	Purpose:
*
*		Get the alignment data we need to align the dialog controls.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _get_dialog_alignment_data(HWND dlg, gui_t* data)
{
	HWND	control;
	RECT	rect;

	control	= GetDlgItem(dlg, IDC_SELECT);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	data->device->margin	= rect.left;

	control	= GetDlgItem(dlg, IDC_TX_ENABLE);
	GetWindowRect(control, &rect);
	ScreenToClient(dlg, (POINT*) &rect.left);
	ScreenToClient(dlg, (POINT*) &rect.right);
	data->device->btn_height	= RECT_HEIGHT(rect) + 1;
}



/*****************************************************************************
*
*	Function:	_align_dialog_content
*
*	Purpose:
*
*		Align the window controls.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _align_dialog_content(HWND dlg, gui_t* data)
{
	static	align_t list[]	=
	{
		// The frame that appears under the menu.
		{// 0
		/* id		*/	IDC_GUI_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_T,
		/* value	*/	0
		},
		{// 1
		/* id		*/	IDC_GUI_FRAME_1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_L,
		/* value	*/	-1
		},
		{// 2
		/* id		*/	IDC_GUI_FRAME_1,
		/* ref		*/	IDC_GPIO_6,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	-999	// See below.
		},
		{// 3
		/* id		*/	IDC_GUI_FRAME_1,
		/* ref		*/	IDC_CABLE_CLOCK,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-999	// See below.
		},

		// The status area.
		{// 4
		/* id		*/	IDC_INFO_RECT,
		/* ref		*/	IDC_CABLE_CLOCK,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	0
		},
		{// 5
		/* id		*/	IDC_INFO_TEXT,
		/* ref		*/	IDC_INFO_RECT,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	-1
		},

		// The frame that surrounds the GUI.
		{// 6
		/* id		*/	IDC_GUI_FRAME_2,
		/* ref		*/	IDC_OPEN,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-999	// See below.
		},
		{// 7
		/* id		*/	IDC_GUI_FRAME_2,
		/* ref		*/	IDC_GUI_FRAME_1,
		/* flags	*/	ALIGN_L_L | ALIGN_B_B | ALIGN_R_R,
		/* value	*/	0
		},

		// The rectangle that is the GUI background.
		{// 8
		/* id		*/	IDC_GUI_RECT_1,
		/* ref		*/	IDC_GUI_FRAME_2,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	1
		},
		{// 9
		/* id		*/	IDC_GUI_RECT_1,
		/* ref		*/	IDC_GUI_FRAME_1,
		/* flags	*/	ALIGN_L_L | ALIGN_B_B | ALIGN_R_R,
		/* value	*/	0
		},

		// The bar under the transmitter.
		{// 10
		/* id		*/	IDC_GUI_RECT_H1,
		/* ref		*/	IDC_GUI_RECT_1,
		/* flags	*/	ALIGN_L_L,
		/* value	*/	0
		},
		{// 11
		/* id		*/	IDC_GUI_RECT_H1,
		/* ref		*/	IDC_TX_SV,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-999	// See below.
		},
		{// 12
		/* id		*/	IDC_GUI_RECT_H1,
		/* ref		*/	IDC_TX_ENABLED_ARROW,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	0
		},
		{// 13
		/* id		*/	IDC_GUI_RECT_H1,
		/* ref		*/	IDC_GUI_RECT_H1,
		/* flags	*/	ALIGN_H,
		/* value	*/	MARGIN
		},

		// The bar under the receiver.
		{// 14
		/* id		*/	IDC_GUI_RECT_H2,
		/* ref		*/	IDC_GUI_RECT_1,
		/* flags	*/	ALIGN_L_L,
		/* value	*/	0
		},
		{// 15
		/* id		*/	IDC_GUI_RECT_H2,
		/* ref		*/	IDC_RX_SV_ARROW,
		/* flags	*/	ALIGN_T_B,
		/* value	*/	-999	// See below.
		},
		{// 16
		/* id		*/	IDC_GUI_RECT_H2,
		/* ref		*/	IDC_RX_SV_ARROW,
		/* flags	*/	ALIGN_R_R,
		/* value	*/	0
		},
		{// 17
		/* id		*/	IDC_GUI_RECT_H2,
		/* ref		*/	IDC_GUI_RECT_H1,
		/* flags	*/	ALIGN_H,
		/* value	*/	MARGIN
		},

		// The bar to the right of the transmitter.
		{// 18
		/* id		*/	IDC_GUI_RECT_V1,
		/* ref		*/	IDC_GUI_RECT_1,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{// 19
		/* id		*/	IDC_GUI_RECT_V1,
		/* ref		*/	IDC_GUI_RECT_H2,
		/* flags	*/	ALIGN_L_R,
		/* value	*/	0
		},
		{// 20
		/* id		*/	IDC_GUI_RECT_V1,
		/* ref		*/	0,
		/* flags	*/	ALIGN_W,
		/* value	*/	MARGIN
		},

		{ 0, 0, 0, 0 }
	};

	HWND	control;
	int		h;
	RECT	rect;
	int		w;

	list[2].value	= data->device->margin + 1;
	list[3].value	= data->device->margin + 1;
	list[6].value	= data->device->margin;
	list[11].value	= data->device->margin;
	list[15].value	= data->device->margin;

	align_control_list(dlg, list);

	control	= GetDlgItem(dlg, IDC_GUI_RECT_V1);
	GetWindowRect(control, &rect);
	h	= rect.bottom - 1;
	h	+= GetSystemMetrics(SM_CYDLGFRAME);
	control	= GetDlgItem(dlg, IDC_CABLE_CLOCK);
	GetWindowRect(control, &rect);
	w	= rect.right;
	w	+= GetSystemMetrics(SM_CXDLGFRAME) + data->device->margin;
	GetWindowRect(dlg, &rect);
	MoveWindow(dlg, rect.left, rect.top, w, h, FALSE);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window's controls per the state of the device
*		in use, if any.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device structure to use.
*
*		info	Any information message to appear in the status box.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	int		i;
	HMENU	menu;
	U32		u32;

	set_win_title(dlg, device, TRUE, TRUE, "GUI");

	if (msg)
		ctrl_menu_set_text(dlg, IDC_INFO_TEXT, msg);

	service_list(dlg, device, _service_list, SERVICE_REFRESH, 0, 0);

	if (device->usable == FALSE)
		;
	else if (device->handle)
		ctrl_menu_set(dlg, IDC_CLOSE, CTRL_FOCUS);
	else
		ctrl_menu_set(dlg, IDC_OPEN, CTRL_FOCUS);

	// Update the menus.
	menu	= GetMenu(dlg);
	menu	= GetSubMenu(menu, 0);
	menu	= GetSubMenu(menu, 0);

	if ((device->usable == FALSE) || (device->handle))
		u32	= MF_GRAYED;
	else
		u32	= MF_ENABLED;

	for (i = 0; i <= 255; i++)
		EnableMenuItem(menu, (UINT) i, MF_BYPOSITION | u32);

	DrawMenuBar(dlg);
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	gui_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		win_pos_save(dlg, SECTION_NAME);
		DestroyWindow(dlg);
		main_close_all(dlg, data->device);
	}
	else
	{
		DestroyWindow(dlg);
	}
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		A service dependent value.
*
*		lParam		A service dependent value.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	gui_t*	data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);
	int		id		= LOWORD(wParam);
	LRESULT	ret		= TRUE;

	switch (id)
	{
		default:

			if ((id >= IDC_SELECT_DEV_0) && (id <= IDC_SELECT_DEV_255))
				select_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			else
				ret	= FALSE;

			break;

		case IDC_ABOUT:

			about_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_C_CODE:

			c_code_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C0:

			cable_command_0_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C1:

			cable_command_1_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C2:

			cable_command_2_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C3:

			cable_command_3_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C4:

			cable_command_4_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C5:

			cable_command_5_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_C6:

			cable_command_6_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CABLE_TRI_STATE:

			misc_tx_rx_tri_state_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CLOSE:

			close_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CFG_FILE_RESTORE:

			config_restore_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CFG_FILE_SAVE:

			config_save_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_CFG_WIZ:

			config_wizard_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_DEVICE_ID:

			device_id_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_READ:

			gpio_read_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO_WRITE:

			gpio_write_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_INIT:

			init_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_MISC_FAVOR_TX:

			misc_favor_tx_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_OPEN:

			open_service(dlg, data->device, SERVICE_WM_COMMAND, FALSE, (ULONG) lParam);
			break;

		case IDC_REGS_FW:

			regs_fw_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_REGS_PCI:

			regs_pci_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_REGS_PLX:

			regs_plx_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_RX_DISABLE:

			rx_disable_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_RX_ENABLE:

			rx_enable_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_RX_FIFO:

			rx_fifo_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_RX_IO:

			rx_io_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_RX_RECEIVE:

			rx_receive_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_SELECT:

			select_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_CD:

			tx_clock_divider_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_OSCILLATOR:

			tx_oscillator_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_DF:

			tx_data_flow_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_DISABLE:

			tx_disable_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_ENABLE:

			tx_enable_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_FIFO:

			tx_fifo_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_IO:

			tx_io_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_LV:

			tx_line_valid_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_SV:

			tx_status_valid_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_TX_TRANSMIT:

			tx_transmit_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDC_VERSION_DATA:

			version_data_service(dlg, data->device, SERVICE_WM_COMMAND, wParam, (ULONG) lParam);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			PostQuitMessage(0);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_ctlcolorstatic
*
*	Purpose:
*
*		Process WM_CTLCOLORSTATIC messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		A service dependent value.
*
*		lParam		A service dependent value.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_ctlcolorstatic(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	HWND	control	= (HWND) lParam;
	gui_t*	data;
	int		id		= GetWindowLong(control, GWL_ID);
	LRESULT	ret;

	switch (id)
	{
		default:

			ret	= 0;
			break;

		case IDC_CABLE:

			data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);
			ret		= cable_service(dlg, data->device, SERVICE_WM_CTLCOLORSTATIC, wParam, (ULONG) lParam);
			break;

		case IDC_GPIO:

			data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);
			ret		= gpio_service(dlg, data->device, SERVICE_WM_CTLCOLORSTATIC, wParam, (ULONG) lParam);
			break;

		case IDC_RX:

			data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);
			ret		= rx_service(dlg, data->device, SERVICE_WM_CTLCOLORSTATIC, wParam, (ULONG) lParam);
			break;

		case IDC_TX:

			data	= (gui_t*) GET_WINDOW_USER_DATA(dlg);
			ret		= tx_service(dlg, data->device, SERVICE_WM_CTLCOLORSTATIC, wParam, (ULONG) lParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the main dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	U32			arg;
	char		buf[128];
	gui_t*		data	= (gui_t*)lParam;
	HDC			dc;
	int			i;
	LOGFONT		lf;
	const char*	msg		= NULL;
	U32			stat;
	TEXTMETRIC	tm;
	int			w;
	int			y;

	SET_WINDOW_USER_DATA(dlg, lParam);

	// Get the fonts that we'll be using.
	dc	= GetDC(dlg);
	y	= GetDeviceCaps(dc, LOGPIXELSY);
	GetTextMetrics(dc, &tm);
	ReleaseDC(dlg, dc);

	memset(&lf, 0, sizeof(lf));
	lf.lfHeight			= -tm.tmHeight;
	lf.lfWidth			= 0;
	lf.lfEscapement		= 0;
	lf.lfOrientation	= 0;
	lf.lfWeight			= tm.tmWeight;
	lf.lfItalic			= 0;
	lf.lfUnderline		= 0;
	lf.lfStrikeOut		= 0;
	lf.lfCharSet		= tm.tmCharSet;
	lf.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	lf.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
	lf.lfQuality		= DEFAULT_QUALITY;
	lf.lfPitchAndFamily	= VARIABLE_PITCH; // tm.tmPitchAndFamily;
	lf.lfFaceName[0]	= 0;

	lf.lfHeight						= -MulDiv(8, y, 72);
	lf.lfWeight						= FW_NORMAL;
	data->device->font.defalt		= CreateFontIndirect(&lf);

	lf.lfHeight						= -MulDiv(8, y, 72);
	lf.lfWeight						= FW_BOLD;
	data->device->font.defalt_b		= CreateFontIndirect(&lf);

	lf.lfHeight						= -MulDiv(10, y, 72);
	lf.lfWeight						= FW_BOLD;
	data->device->font.defalt_bl	= CreateFontIndirect(&lf);

	lf.lfHeight						= -MulDiv(12, y, 72);
	lf.lfWeight						= FW_BOLD;
	data->device->font.defalt_bll	= CreateFontIndirect(&lf);

	lf.lfHeight						= -MulDiv(8, y, 72);
	lf.lfPitchAndFamily				= FIXED_PITCH;
	lf.lfWeight						= FW_NORMAL;
	data->device->font.fixed		= CreateFontIndirect(&lf);

	lf.lfHeight						= -MulDiv(10, y, 72);
	lf.lfPitchAndFamily				= FIXED_PITCH;
	lf.lfWeight						= FW_NORMAL;
	data->device->font.fixed_l		= CreateFontIndirect(&lf);

	// Verify the state of the API.
	data->device->api_status	= hpdi32_api_status(&stat,
													&arg,
													HPDI32_API_VERSION);

	if (data->device->api_status == GSC_SUCCESS)
		data->device->api_status	= stat;

	if (data->device->api_status == GSC_SUCCESS)
		data->device->usable	= TRUE;

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	if (data->device->usable)
	{
		msg	= buf;

		if (data->device->usable == FALSE)
			buf[0]	= 0;
		else if (data->device->handle)
			sprintf(buf, "Accessing device #%d.", data->device->index);
		else
			sprintf(buf, "Device count: %d", (int) data->device->count);
	}

	_get_dialog_alignment_data(dlg, data);
	service_list(dlg, data->device, _service_list, SERVICE_INIT, 0, 0);
	_align_dialog_content(dlg, data);

	w	= static_text_width_get(dlg, IDC_TX_CLOCK);
	i	= static_text_width_get(dlg, IDC_TX_RX_READY);
	w	= (w < i) ? i : w;
	i	= static_text_width_get(dlg, IDC_TX_DR);
	w	= (w < i) ? i : w;
	i	= static_text_width_get(dlg, IDC_TX_FV);
	w	= (w < i) ? i : w;
	i	= static_text_width_get(dlg, IDC_TX_ENABLED);
	w	= (w < i) ? i : w;

	i	= button_text_width_get(dlg, IDC_TX_DF);
	w	= (w < i) ? i : w;
	i	= button_text_width_get(dlg, IDC_TX_LV);
	w	= (w < i) ? i : w;
	i	= button_text_width_get(dlg, IDC_TX_SV);
	w	= (w < i) ? i : w;

	w	+= MARGIN * 2;
	static_arrow_offset_set(dlg, IDC_TX_CLOCK, w);
	static_arrow_offset_set(dlg, IDC_TX_DR, w);
	static_arrow_offset_set(dlg, IDC_TX_ENABLED, w);
	static_arrow_offset_set(dlg, IDC_TX_FV, w);
	static_arrow_offset_set(dlg, IDC_TX_RX_READY, w);

	button_arrow_offset_set(dlg, IDC_TX_DF, w);
	button_arrow_offset_set(dlg, IDC_TX_LV, w);
	button_arrow_offset_set(dlg, IDC_TX_SV, w);

	static_arrow_offset_set(dlg, IDC_RX_CLOCK, w);
	static_arrow_offset_set(dlg, IDC_RX_DATA, w);
	static_arrow_offset_set(dlg, IDC_RX_FV, w);
	static_arrow_offset_set(dlg, IDC_RX_LV, w);
	static_arrow_offset_set(dlg, IDC_RX_SV, w);
	static_arrow_offset_set(dlg, IDC_RX_RR, w);
	static_arrow_offset_set(dlg, IDC_RX_ENABLED, w);

	static_arrow_offset_set(dlg, IDC_GPIO_0, w);
	static_arrow_offset_set(dlg, IDC_GPIO_1, w);
	static_arrow_offset_set(dlg, IDC_GPIO_2, w);
	static_arrow_offset_set(dlg, IDC_GPIO_3, w);
	static_arrow_offset_set(dlg, IDC_GPIO_4, w);
	static_arrow_offset_set(dlg, IDC_GPIO_5, w);
	static_arrow_offset_set(dlg, IDC_GPIO_6, w);

	win_pos_restore(dlg, SECTION_NAME, FALSE);
	main_refresh_add(dlg, _refresh_window);
	main_refresh(data->device, NULL);
	ctrl_menu_set_text(dlg, IDC_INFO_TEXT, msg);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the main dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			PostQuitMessage(0);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_CTLCOLORSTATIC:

			ret	= _wm_ctlcolorstatic(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_DRAWITEM:

			if (wParam)
				ret	= button_wm_drawitem((LPDRAWITEMSTRUCT) lParam);
			else
				ret	= FALSE;

			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		All went well.
*		1		There was a problem.
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static gui_t	data;
	LRESULT			ret;

	data.device		= device;

	dlg		= CreateDialogParam(device->instance,
								MAKEINTRESOURCE(IDD_GUI),
								NULL,
								(DLGPROC) _dlg_proc,
								(LPARAM) &data);
	ShowWindow(dlg, SW_SHOWNORMAL);
	UpdateWindow(dlg);
	SetFocus(dlg);
	ret	= dlg ? 0 : 1;
	return(ret);
}



/*****************************************************************************
*
*	Function:	gui_service
*
*	Purpose:
*
*		Implement the common service entry point for the GUI services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gui_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	LRESULT	ret;

	if (service == SERVICE_WM_COMMAND)
		ret	= _service_wm_command(dlg, device, wParam, lParam);
	else
		ret	= 0;

	return(ret);
}


