// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/io.c $
// $Rev: 31050 $
// $Date: 2014-10-27 17:51:19 -0500 (Mon, 27 Oct 2014) $

/*****************************************************************************
*
*	Implement support for common I/O services.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "io.h"
#include "copy.h"
#include "gui.h"
#include "ini.h"
#include "preference.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	KEY_APP_BUFFER		"app buffer"
#define	KEY_API_BUFFER_SIZE	"api buffer size"



/* prototypes	*************************************************************/

static	U32	_dma_ch_sel_cfg_wiz(HWND				dlg,
								device_t*			device,
								const parm_entry_t*	entry,
								U32					value);



/* variables	*************************************************************/

static const ctrl_id_u32_t	_mode_list[]	=
{
	/* ctrl_id				u32						*/
	{ IDC_IO_MODE_PIO,		HPDI32_IO_MODE_PIO		},
	{ IDC_IO_MODE_DMA,		HPDI32_IO_MODE_DMA		},
	{ IDC_IO_MODE_DMDMA,	HPDI32_IO_MODE_DMDMA	}
};

static const ctrl_id_u32_t	_dma_sel_list[]	=
{
	/* ctrl_id								u32									*/
	{ IDC_IO_DMA_CHANNEL_SELECT_STATIC,		HPDI32_IO_DMA_CHANNEL_SEL_STATIC	},
	{ IDC_IO_DMA_CHANNEL_SELECT_DYNAMIC,	HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC	}
};

static const ctrl_id_u32_t	_dma_pri_list[]	=
{
	/* ctrl_id						u32								*/
	{ IDC_IO_DMA_PRIORITY_DISABLE,	HPDI32_IO_DMA_PRIORITY_DISABLE	},
	{ IDC_IO_DMA_PRIORITY_ENABLE,	HPDI32_IO_DMA_PRIORITY_ENABLE	}
};

static const ctrl_id_u32_t	_dma_ctrl_list[]	=
{
	/* ctrl_id						u32										*/
	{ IDC_IO_DMA_CONTROL_MANUAL,	HPDI32_IO_DMA_CONTROL_MODE_MANUAL		},
	{ IDC_IO_DMA_CONTROL_AUTO,		HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC	}
};

static const ctrl_id_u32_t	_single_cycle_list[]	=
{
	/* ctrl_id						u32								*/
	{ IDC_IO_SINGLE_CYCLE_ABSENT,	HPDI32_IO_SINGLE_CYCLE_ABSENT	},
	{ IDC_IO_SINGLE_CYCLE_PRESENT,	HPDI32_IO_SINGLE_CYCLE_PRESENT	}
};

static const ctrl_id_u32_t	_data_size_list[]	=
{
	/* ctrl_id				u32							*/
	{ IDC_IO_DATA_SIZE_8,	HPDI32_IO_DATA_SIZE_8_BITS	},
	{ IDC_IO_DATA_SIZE_16,	HPDI32_IO_DATA_SIZE_16_BITS	},
	{ IDC_IO_DATA_SIZE_32,	HPDI32_IO_DATA_SIZE_32_BITS	}
};

static const ctrl_id_u32_t	_app_buf_list[]	=
{
	/* ctrl_id					u32	*/
	{ IDC_IO_XFER_BUFFER_APP,	1	},
	{ IDC_IO_XFER_BUFFER_API,	0	}
};

static const value_list_entry_t	_mode_value_list[]	=
{
	{
	/* user_text			*/	"Demand Mode DMA (Offers the highest"
								" potential throughput and can transfer very"
								" large blocks of data.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_MODE_DMDMA),
	/* suffix				*/	"_DMDMA"
	},
	{
	/* user_text			*/	"DMA (The API or the application must examine"
								" the FIFO status before performing transfers.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_MODE_DMA),
	/* suffix				*/	"_DMA"
	},
	{
	/* user_text			*/	"PIO (Data transfer uses repetative register"
								" accesses.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_MODE_PIO),
	/* suffix				*/	"_PIO"
	}
};

static const value_list_t	_mode_values[]	=
{
	{
	/* list	*/	_mode_value_list,
	/* size	*/	ARRAY_ELEMENTS(_mode_value_list)
	}
};

static const value_t	_mode_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_mode_values
	}
};

static const parm_entry_t	_mode	=
{
	/* title			*/	"Mode",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_MODE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_mode_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls how data is transferred across the PCI bus"
	" between host memory and the HPDI32 during I/O requests. This is a major"
	" factor affecting throughput rates."
};

static const value_list_entry_t	_dma_ch_sel_value_list[]	=
{
	{
	/* user_text			*/	"Static (DMA channels are acquired and"
								" released only when needed.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CHANNEL_SEL_STATIC),
	/* suffix				*/	"_STATIC"
	},
	{
	/* user_text			*/	"Dynamic (DMA channels are acquired when"
								" needed and are release as soon as possible.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC),
	/* suffix				*/	"_DYNAMIC"
	}
};

static const value_list_t	_dma_ch_sel_values[]	=
{
	{
	/* list	*/	_dma_ch_sel_value_list,
	/* size	*/	ARRAY_ELEMENTS(_dma_ch_sel_value_list)
	}
};

static const value_t	_dma_ch_sel_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_dma_ch_sel_values
	}
};

static const parm_entry_t	_dma_ch_sel	=
{
	/* title			*/	"DMA Channel Select",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CHANNEL_SEL),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_dma_ch_sel_value,
	/* cfg_wiz			*/	_dma_ch_sel_cfg_wiz,
	/* desc				*/
	"This parameter controls the API's acquisition and release of a DMA"
	" channel for DMA based I/O. A DMA channel is acquired at the start of"
	" each DMA based I/O request. The acquisition effort is greater if a DMA"
	" channel has not already been acquired for that I/O direction"
	" (read/write). If called for, the Dynamic option, then the API will"
	" release the DMA channel when the I/O call completes. Each acquisition"
	" and release effort takes additional overhead and contributes to reduced"
	" throughput. The Static option generally offers the highest potential"
	" throughput."
	" NOTE: This parameter can be ignored if DMA is not used for I/O requests."
	" NOTE: This parameter should be set to Dynamic if using Demand Mode DMA"
	" for simultaneous read/write operations to the FIFOs on a board that"
	" does not support the use of Demand Mode DMA on DMA Channel 1."
};

static const value_list_entry_t	_dma_priority_value_list[]	=
{
	{
	/* user_text			*/	"Enable (Request preferential PCI bus access."
								" This is the preferred option.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_PRIORITY_ENABLE),
	/* suffix				*/	"_ENABLE"
	},
	{
	/* user_text			*/	"Disable (Don't request preferential PCI bus access.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_PRIORITY_DISABLE),
	/* suffix				*/	"_DISABLE"
	}
};

static const value_list_t	_dma_priority_values[]	=
{
	{
	/* list	*/	_dma_priority_value_list,
	/* size	*/	ARRAY_ELEMENTS(_dma_priority_value_list)
	}
};

static const value_t	_dma_priority_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_dma_priority_values
	}
};

static const parm_entry_t	_dma_priority	=
{
	/* title			*/	"DMA Priority",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_PRIORITY),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_dma_priority_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls how the HPDI32 DMA engine prioritizes data"
	" movement across the PCI bus when performing simultaneous DMA transfers"
	" both to and from the HPDI32. If either Tx or Rx is given priority, not"
	" both, then it gets preferential use of the PCI bus when simultaneous"
	" DMA is in progress. If both have the same priority (i.e. both are"
	" enabled or disabled), then neither gets preference of the bus during"
	" simultaneous DMA transfers."
	" NOTE: This parameter can be ignored if DMA is not used for I/O requests"
	" or if performing transmit only or receive only operations."
};

static const value_list_entry_t	_dma_control_mode_value_list[]	=
{
	{
	/* user_text			*/	"Automatic (Requires less work by the"
								" application as API checks the FIFO status.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC),
	/* suffix				*/	"_AUTO"
	},
	{
	/* user_text			*/	"Manual (Requires more work by the"
								" application, but may give better throughput.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CONTROL_MODE_MANUAL),
	/* suffix				*/	"_MANUAL"
	}
};

static const value_list_t	_dma_control_mode_values[]	=
{
	{
	/* list	*/	_dma_control_mode_value_list,
	/* size	*/	ARRAY_ELEMENTS(_dma_control_mode_value_list)
	}
};

static const value_t	_dma_control_mode_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_dma_control_mode_values
	}
};

static const parm_entry_t	_dma_control_mode	=
{
	/* title			*/	"DMA Control Mode (for non-Demand Mode DMA)",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_DMA_CONTROL_MODE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_dma_control_mode_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls how the API performs data transfers for"
	" non-Demand Mode based DMA I/O requests. For non-Demand Mode DMA, the"
	" HPDI32 DMA engine performs the transfer without regard to the FIFO fill"
	" level -- the transfer is performed blindly. Reads will return"
	" indeterminate data if the Rx FIFO underflows and writes will loose data"
	" if the Tx FIFO overflows. Host software then, either the application or"
	" the API, must limit the size of I/O requests in order to prevent"
	" underflows and overflows."
	" NOTE: This parameter can be ignored if using either PIO or Demand Mode"
	" DMA."
};

static const value_list_entry_t	_single_cycle_value_list[]	=
{
	{
	/* user_text			*/	"Absent (When appropriate, firmware pauses"
								" DMDMA.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_SINGLE_CYCLE_ABSENT),
	/* suffix				*/	"_ABSENT"
	},
	{
	/* user_text			*/	"Present (When appropriate, firmware slows"
								" DMDMA.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_SINGLE_CYCLE_PRESENT),
	/* suffix				*/	"_PRESENT"
	}
};

static const value_list_t	_single_cycle_values[]	=
{
	{
	/* list	*/	_single_cycle_value_list,
	/* size	*/	ARRAY_ELEMENTS(_single_cycle_value_list)
	}
};

static const value_t	_single_cycle_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_single_cycle_values
	}
};

static const parm_entry_t	_single_cycle	=
{
	/* title			*/	"Single Cycle",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_SINGLE_CYCLE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_single_cycle_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter relates to the transfer of data between the FIFOs and the"
	" DMA engines, but only under particular circumstances. 1) It applies only"
	" to Demand Mode DMA transfers. 2) It applies only when the data size (in"
	" bits) is less than the PCI bus size (in bits). 3) It applies only to"
	" those boards that do not support the Single Cycle Disable feature. In"
	" all other circumstances the API ignores this parameter. Applications"
	" should otherwise set this parameter according to how their HPDI32 responds"
	" when the Tx FIFO becomes Almost Full or the Rx FIFO becomes Almost"
	" Empty. If firmware PAUSES data transfer at these points, then applications"
	" should select the ABSENT option. This should permit higher thoughput"
	" rates. If firmware SLOWS data transfer at these points (using Single Cycle"
	" accesses to the FIFO), then this parameter should be set to PRESENT. If"
	" not, then the Tx data stream may loose some data samples and the Rx data"
	" stream may include extra data samples. Overall troughput may likely be"
	" lower, but data integrity is maintained."
};

static const value_list_entry_t	_data_size_value_list[]	=
{
	{
	/* user_text			*/	"32-bits (Use all 32-bits of the cable.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DATA_SIZE_32_BITS),
	/* suffix				*/	"_32"
	},
	{
	/* user_text			*/	"16-bits (Use the lower 16-bits of the cable.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DATA_SIZE_16_BITS),
	/* suffix				*/	"_16"
	},
	{
	/* user_text			*/	"8-bits (Use the lower 8-bits of the cable.)",
	/* value + value_text	*/	VAR_STR_MACRO(HPDI32_IO_DATA_SIZE_8_BITS),
	/* suffix				*/	"_8"
	}
};

static const value_list_t	_data_size_values[]	=
{
	{
	/* list	*/	_data_size_value_list,
	/* size	*/	ARRAY_ELEMENTS(_data_size_value_list)
	}
};

static const value_t	_data_size_value	=
{
	/* type	*/	VALUE_TYPE_LIST,
	{
	/* u.vp	*/	_data_size_values
	}
};

static const parm_entry_t	_data_size	=
{
	/* title			*/	"Data Size",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_DATA_SIZE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_data_size_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter specifies the size of the data being transferred across"
	" the cable interface. FIFO sizes refer to the number of samples they"
	" hold, not the number of bytes. The FIFOs will not hold more samples by"
	" selecting a smaller data size."
};

static const value_range_t	_buffer_size_value_range	=
{
	/* min	*/	0,
	/* max	*/	GSC_IO_STATUS_COUNT_MASK,
	/* k	*/	1024
};

static const value_t	_buffer_size_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_buffer_size_value_range
	}
};

static const parm_entry_t	_buffer_size	=
{
	/* title			*/	"API Buffer Size",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_BUFFER_SIZE),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	", NULL",
	/* get_arg			*/	", &api_buf_size",
	/* values			*/	&_buffer_size_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls the use of and the size of the API Buffer that"
	" the API will try to allocate. The API Buffers are physically contiguous"
	" memory regions usable by the Device Driver, the API and the"
	" application, and are provided primarily to facilitate more efficient"
	" DMA based I/O requests. Because physically contiguous memory regions"
	" are a limited resource, the OS may not be able to fulfill the request"
	" as given. The size of the region granted by the OS may therefore be"
	" smaller than requested. Applications wishing to use API Buffers must do"
	" so explicitly (see the user manual and the Generate C Source Code"
	" output). The API maintains separate Tx and Rx API Buffers, but"
	" applications can ignore this association and may use them as desired."
	" If API Buffers are used, applications are free to use either API"
	" Buffers or Application Buffers for I/O requests on an as needed basis."
	" However, I/O operations are less efficient when switching between one"
	" buffer type and the other when using DMA based I/O. NOTE: This"
	" parameter should be set to zero (0x0) if the application will not be"
	" using API Buffers."
};

static const value_range_t	_pio_threshold_value_range	=
{
	/* min	*/	0,
	/* max	*/	GSC_IO_STATUS_COUNT_MASK,
	/* k	*/	1024
};

static const value_t	_pio_threshold_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_pio_threshold_value_range
	}
};

static const parm_entry_t	_pio_threshold	=
{
	/* title			*/	"PIO Threshold",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_PIO_THRESHOLD),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_pio_threshold_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter controls if and when the API will automatically revert to"
	" PIO mode data transfers should a DMA I/O request be too small. DMA"
	" transfers require overhead to acquire, setup, use and release a DMA"
	" channel. For small requests this overhead can exceed the benefit of"
	" using DMA. If the number of samples in an I/O request is at or below"
	" this level, then the API will quietly perform the transfer using PIO"
	" rather than DMA."
	" NOTE: This parameter can be ignored if not using DMA."
};

static const value_range_t	_timeout_value_range	=
{
	/* min	*/	0,
	/* max	*/	HPDI32_IO_TIMEOUT_MAX,
	/* k	*/	1000
};

static const value_t	_timeout_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_timeout_value_range
	}
};

static const parm_entry_t	_timeout	=
{
	/* title			*/	"Timeout",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_TIMEOUT),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	"",
	/* get_arg			*/	"",
	/* values			*/	&_timeout_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/
	"This parameter sets the limit for the amount of time the API can spend"
	" in completing an I/O request. If an I/O request doesn't complete within"
	" this time limit, then the request is aborted. For PIO and DMA, an abort"
	" generally has no negative consequences in that the number of bytes"
	" reported to have been transferred should be accurate. For an aborted"
	" Demand Mode DMA request however, the amount of data actually"
	" transferred is unknown. Applications should set the timeout limit so"
	" that all requests will complete without timing out. This way, a timeout"
	" can be used to indicate that something has gone wrong. A timeout value"
	" of zero tells the API that it should abort an I/O request prematurely"
	" if it would otherwise have to wait for completion. A value of zero"
	" should generally not be used with DMA."
};

const parm_entry_t*	io_parm_list[9]	=
{
	&_mode,
	&_dma_ch_sel,
	&_dma_priority,
	&_dma_control_mode,
	&_data_size,
	&_single_cycle,
	&_buffer_size,
	&_pio_threshold,
	&_timeout
};

static const value_range_t	_buffer_pointer_value_range	=
{
	/* min	*/	0,
	/* max	*/	0xFFFFFFFFUL,
	/* k	*/	1024
};

static const value_t	_buffer_pointer_value	=
{
	/* type	*/	VALUE_TYPE_RANGE,
	{
	/* u.vp	*/	&_buffer_pointer_value_range
	}
};

static const parm_entry_t	_buffer_pointer	=
{
	/* title			*/	"",
	/* parm + parm_text	*/	VAR_STR_MACRO(HPDI32_IO_BUFFER_POINTER),
	/* suffix			*/	"",
	/* which			*/	0,
	/* set_arg			*/	", NULL",
	/* get_arg			*/	", (void*) &api_buf_ptr",
	/* values			*/	&_buffer_pointer_value,
	/* cfg_wiz			*/	NULL,
	/* desc				*/	NULL
};

const parm_entry_t*	io_parm_api_buffer_list[2]	=
{
	&_buffer_pointer,
	&_buffer_size
};



/*****************************************************************************
*
*	Function:	_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_MODE__GET(	data->device->handle,
									data->which,
									&data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_MODE__SET(	data->device->handle,
									data->which,
									data->mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_dialog_read(HWND dlg, io_data_t* data)
{
	data->mode.setting	= ctrl_radio_2_u32(	dlg,
											_mode_list,
											ARRAY_ELEMENTS(_mode_list),
											data->mode.defalt);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _mode_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		u32		= data->mode.setting;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _mode_list, ARRAY_ELEMENTS(_mode_list));
	ctrl_menu_set(dlg, IDC_IO_MODE, flags);
	ctrl_menu_set(dlg, IDC_IO_MODE_DMDMA, flags);
	ctrl_menu_set(dlg, IDC_IO_MODE_DMA, flags);
	ctrl_menu_set(dlg, IDC_IO_MODE_PIO, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_dma_ch_sel_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_ch_sel_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_CHANNEL_SEL__GET(	data->device->handle,
												data->which,
												&data->dma_ch_sel.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_ch_sel_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_ch_sel_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_CHANNEL_SEL__SET(	data->device->handle,
												data->which,
												data->dma_ch_sel.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_ch_sel_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_ch_sel_dialog_read(HWND dlg, io_data_t* data)
{
	data->dma_ch_sel.setting	= ctrl_radio_2_u32(
											dlg,
											_dma_sel_list,
											ARRAY_ELEMENTS(_dma_sel_list),
											data->dma_ch_sel.defalt);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_dma_ch_sel_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_ch_sel_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		u32	= data->dma_ch_sel.setting;

		if (data->mode.setting == HPDI32_IO_MODE_PIO)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _dma_sel_list, ARRAY_ELEMENTS(_dma_sel_list));
	ctrl_menu_set(dlg, IDC_IO_DMA_CHANNEL_SELECT_MODE, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_CHANNEL_SELECT_STATIC, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_CHANNEL_SELECT_DYNAMIC, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_dma_priority_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_priority_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_PRIORITY__GET(	data->device->handle,
											data->which,
											&data->dma_pri.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_priority_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_priority_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_PRIORITY__SET(	data->device->handle,
											data->which,
											data->dma_pri.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_priority_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_priority_dialog_read(HWND dlg, io_data_t* data)
{
	data->dma_pri.setting	= ctrl_radio_2_u32(	dlg,
												_dma_pri_list,
												ARRAY_ELEMENTS(_dma_pri_list),
												data->dma_pri.defalt);
	return(GSC_SUCCESS);
}




/*****************************************************************************
*
*	Function:	_dma_priority_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_priority_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		u32	= data->dma_pri.setting;

		if (data->mode.setting == HPDI32_IO_MODE_PIO)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _dma_pri_list, ARRAY_ELEMENTS(_dma_pri_list));
	ctrl_menu_set(dlg, IDC_IO_DMA_PRIORITY, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_PRIORITY_ENABLE, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_PRIORITY_DISABLE, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_dma_control_mode_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_control_mode_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_CONTROL_MODE__GET(	data->device->handle,
												data->which,
												&data->dma_ctrl_mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_control_mode_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_control_mode_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DMA_CONTROL_MODE__SET(	data->device->handle,
												data->which,
												data->dma_ctrl_mode.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_dma_control_mode_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_control_mode_dialog_read(HWND dlg, io_data_t* data)
{
	data->dma_ctrl_mode.setting	= ctrl_radio_2_u32(
										dlg,
										_dma_ctrl_list,
										ARRAY_ELEMENTS(_dma_ctrl_list),
										data->dma_ctrl_mode.defalt);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_dma_control_mode_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_control_mode_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		u32	= data->dma_ctrl_mode.setting;

		if (data->mode.setting == HPDI32_IO_MODE_DMA)
			flags	= CTRL_ENABLE;
		else
			flags	= CTRL_DISABLE;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _dma_ctrl_list, ARRAY_ELEMENTS(_dma_ctrl_list));
	ctrl_menu_set(dlg, IDC_IO_DMA_CONTROL_MODE, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_CONTROL_MANUAL, flags);
	ctrl_menu_set(dlg, IDC_IO_DMA_CONTROL_AUTO, flags);

	return(GSC_SUCCESS);
}




/*****************************************************************************
*
*	Function:	_single_cycle_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _single_cycle_device_read(io_data_t* data)
{
	U32	stat;
	U32	status;

	status	= HPDI32_IO_SINGLE_CYCLE__GET(	data->device->handle,
											data->which,
											&data->single_cycle.setting);
	stat	= HPDI32_MISC_FEATURES__1_CYCLE_DISABLE(data->device->handle,
													&data->single_cycle_disable);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_FEATURES__1_CYCLE_DISABLE(data->device->handle,
													&data->single_cycle_disable);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= HPDI32_MISC_PCI_BUS_WIDTH__GET(	data->device->handle,
												&data->bus_size);
	status	= (status == GSC_SUCCESS) ? stat : status;
	return(status);
}



/*****************************************************************************
*
*	Function:	_single_cycle_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _single_cycle_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_SINGLE_CYCLE__SET(	data->device->handle,
											data->which,
											data->single_cycle.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_single_cycle_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _single_cycle_dialog_read(HWND dlg, io_data_t* data)
{
	data->single_cycle.setting	= ctrl_radio_2_u32(
										dlg,
										_single_cycle_list,
										ARRAY_ELEMENTS(_single_cycle_list),
										data->single_cycle.defalt);
	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_single_cycle_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _single_cycle_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		if (data->mode.setting != HPDI32_IO_MODE_DMDMA)
			flags	= CTRL_DISABLE;
		else if (data->single_cycle_disable == HPDI32_MISC_FEATURES_PRESENT)
			flags	= CTRL_DISABLE;
		else if (data->data_size.setting >= data->bus_size)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;

		u32	= data->single_cycle.setting;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(	dlg,
						u32,
						_single_cycle_list,
						ARRAY_ELEMENTS(_single_cycle_list));
	ctrl_menu_set(dlg, IDC_IO_SINGLE_CYCLE, flags);
	ctrl_menu_set(dlg, IDC_IO_SINGLE_CYCLE_ABSENT, flags);
	ctrl_menu_set(dlg, IDC_IO_SINGLE_CYCLE_PRESENT, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_data_size_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _data_size_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DATA_SIZE__GET(	data->device->handle,
										data->which,
										&data->data_size.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_data_size_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _data_size_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_DATA_SIZE__SET(	data->device->handle,
										data->which,
										data->data_size.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_data_size_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _data_size_dialog_read(HWND dlg, io_data_t* data)
{
	data->data_size.setting	= ctrl_radio_2_u32(
											dlg,
											_data_size_list,
											ARRAY_ELEMENTS(_data_size_list),
											data->data_size.defalt);
	return(GSC_SUCCESS);
}




/*****************************************************************************
*
*	Function:	_data_size_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _data_size_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		u32		= data->data_size.setting;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _data_size_list, ARRAY_ELEMENTS(_data_size_list));
	ctrl_menu_set(dlg, IDC_IO_DATA_SIZE, flags);
	ctrl_menu_set(dlg, IDC_IO_DATA_SIZE_8, flags);
	ctrl_menu_set(dlg, IDC_IO_DATA_SIZE_16, flags);
	ctrl_menu_set(dlg, IDC_IO_DATA_SIZE_32, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_buffer_type_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _buffer_type_device_read(io_data_t* data)
{
	U32	u32;
	U32	status;

	status	= HPDI32_IO_BUFFER_SIZE__GET(	data->device->handle,
											data->which,
											&u32);
	data->app_buf.setting	= u32 ? 0 : 1;

	if (data->which & HPDI32_WHICH_TX)
		data->device->tx.app_buf	= (int) data->app_buf.setting;
	else
		data->device->rx.app_buf	= (int) data->app_buf.setting;

	return(status);
}



/*****************************************************************************
*
*	Function:	_buffer_type_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _buffer_type_device_write(io_data_t* data)
{
	if (data->which & HPDI32_WHICH_TX)
		data->device->tx.app_buf	= (int) data->app_buf.setting;
	else
		data->device->rx.app_buf	= (int) data->app_buf.setting;

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_buffer_type_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _buffer_type_dialog_read(HWND dlg, io_data_t* data)
{
	data->app_buf.setting	= ctrl_radio_2_u32(
											dlg,
											_app_buf_list,
											ARRAY_ELEMENTS(_app_buf_list),
											data->app_buf.defalt);
	return(GSC_SUCCESS);
}




/*****************************************************************************
*
*	Function:	_buffer_type_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _buffer_type_dialog_write(HWND dlg, io_data_t* data)
{
	unsigned int	flags;
	U32				u32;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		u32		= data->app_buf.setting;
	}
	else
	{
		flags	= CTRL_DISABLE;
		u32		= (U32) -1;
	}

	ctrl_u32_2_radio(dlg, u32, _app_buf_list, ARRAY_ELEMENTS(_app_buf_list));
	ctrl_menu_set(dlg, IDC_IO_XFER_BUFFER, flags);
	ctrl_menu_set(dlg, IDC_IO_XFER_BUFFER_APP, flags);
	ctrl_menu_set(dlg, IDC_IO_XFER_BUFFER_API, flags);

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_api_buffer_size_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _api_buffer_size_device_read(io_data_t* data)
{
	U32	u32;
	U32	status;

	status	= HPDI32_IO_BUFFER_SIZE__GET(	data->device->handle,
											data->which,
											&u32);

	if (u32)
	{
		if (data->which & HPDI32_WHICH_TX)
			data->device->tx.buf_size	= u32;
		else
			data->device->rx.buf_size	= u32;
	}

	if (data->which & HPDI32_WHICH_TX)
		data->api_buf_size.setting	= data->device->tx.buf_size;
	else
		data->api_buf_size.setting	= data->device->rx.buf_size;

	return(status);
}



/*****************************************************************************
*
*	Function:	_api_buffer_size_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _api_buffer_size_device_write(io_data_t* data)
{
	HCURSOR	cursor;
	U32		size;
	U32		status;

	if (data->which & HPDI32_WHICH_TX)
		data->device->tx.buf_size	= data->api_buf_size.setting;
	else
		data->device->rx.buf_size	= data->api_buf_size.setting;

	if (data->app_buf.setting)
		size	= 0;
	else
		size	= data->api_buf_size.setting;

	cursor	= SetCursor(data->device->cursor.wait);
	status	= HPDI32_IO_BUFFER_SIZE__SET(	data->device->handle,
											data->which,
											size,
											NULL);
	SetCursor(cursor);
	return(status);
}



/*****************************************************************************
*
*	Function:	_api_buffer_size_dialog_read
*
*	Purpose:
*
*		Read the parameter from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _api_buffer_size_dialog_read(HWND dlg, io_data_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_IO_API_BUFFER_SIZE_EDIT,
								_buffer_size_value_range.k,
								&data->api_buf_size.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_api_buffer_size_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _api_buffer_size_dialog_write(HWND dlg, io_data_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		if (data->app_buf.setting)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;

		format_u32(	data->api_buf_size.setting,
					_buffer_size_value_range.k,
					buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_IO_API_BUFFER_SIZE, flags);
	ctrl_menu_set(dlg, IDC_IO_API_BUFFER_SIZE_EDIT, flags);
	ctrl_menu_set(dlg, IDC_IO_API_BUFFER_SIZE_TEXT, flags);

	if (data->api_buf_size.ignore == FALSE)
	{
		data->api_buf_size.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_IO_API_BUFFER_SIZE_EDIT, buf);
		data->api_buf_size.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_pio_threshold_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _pio_threshold_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_PIO_THRESHOLD__GET(	data->device->handle,
											data->which,
											&data->pio_threshold.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_pio_threshold_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _pio_threshold_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_PIO_THRESHOLD__SET(	data->device->handle,
											data->which,
											data->pio_threshold.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_pio_threshold_dialog_read
*
*	Purpose:
*
*		Retrieve the current I/O PIO Threshold setting from the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _pio_threshold_dialog_read(HWND dlg, io_data_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_IO_PIO_THRESHOLD_EDIT,
								_pio_threshold_value_range.k,
								&data->pio_threshold.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_pio_threshold_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _pio_threshold_dialog_write(HWND dlg, io_data_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		format_u32(	data->pio_threshold.setting,
					_pio_threshold_value_range.k,
					buf);

		if (data->mode.setting == HPDI32_IO_MODE_PIO)
			flags	= CTRL_DISABLE;
		else
			flags	= CTRL_ENABLE;
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_IO_PIO_THRESHOLD, flags);
	ctrl_menu_set(dlg, IDC_IO_PIO_THRESHOLD_EDIT, flags);
	ctrl_menu_set(dlg, IDC_IO_PIO_THRESHOLD_TEXT, flags);

	if (data->pio_threshold.ignore == FALSE)
	{
		data->pio_threshold.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_IO_PIO_THRESHOLD_EDIT, buf);
		data->pio_threshold.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_timeout_device_read
*
*	Purpose:
*
*		Read the parameter from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _timeout_device_read(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_TIMEOUT__GET(	data->device->handle,
										data->which,
										&data->timeout.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_timeout_device_write
*
*	Purpose:
*
*		Write the parameter held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _timeout_device_write(io_data_t* data)
{
	U32	status;

	status	= HPDI32_IO_TIMEOUT__SET(	data->device->handle,
										data->which,
										data->timeout.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_timeout_dialog_read
*
*	Purpose:
*
*		Retrieve the current I/O Timeout setting from the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _timeout_dialog_read(HWND dlg, io_data_t* data)
{
	U32	status;

	status	= ctrl_text_2_u32(	dlg,
								IDC_IO_TIMEOUT_EDIT,
								_timeout_value_range.k,
								&data->timeout.setting);
	return(status);
}



/*****************************************************************************
*
*	Function:	_timeout_dialog_write
*
*	Purpose:
*
*		Write the parameter held in memory to the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _timeout_dialog_write(HWND dlg, io_data_t* data)
{
	char			buf[32];
	unsigned int	flags;

	if (data->device->handle)
	{
		flags	= CTRL_ENABLE;
		format_u32(data->timeout.setting, _timeout_value_range.k, buf);
	}
	else
	{
		flags	= CTRL_DISABLE;
		buf[0]	= 0;
	}

	ctrl_menu_set(dlg, IDC_IO_TIMEOUT, flags);
	ctrl_menu_set(dlg, IDC_IO_TIMEOUT_EDIT, flags);
	ctrl_menu_set(dlg, IDC_IO_TIMEOUT_TEXT, flags);

	if (data->timeout.ignore == FALSE)
	{
		data->timeout.ignore	= TRUE;
		ctrl_menu_set_text(dlg, IDC_IO_TIMEOUT_EDIT, buf);
		data->timeout.ignore	= FALSE;
	}

	return(GSC_SUCCESS);
}



/*****************************************************************************
*
*	Function:	_device_read
*
*	Purpose:
*
*		Read the parameters from the device into memory.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_read(io_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_ch_sel_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_priority_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_control_mode_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _single_cycle_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _data_size_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _buffer_type_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _api_buffer_size_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _pio_threshold_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _timeout_device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_device_write
*
*	Purpose:
*
*		Write the parameters held in memory to the device.
*
*	Arguments:
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _device_write(io_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_ch_sel_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_priority_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_control_mode_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _single_cycle_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _data_size_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _buffer_type_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _api_buffer_size_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _pio_threshold_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _timeout_device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_dialog_read
*
*	Purpose:
*
*		Read the parameters from the dialog into memory.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_read(HWND dlg, io_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_ch_sel_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_priority_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_control_mode_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _single_cycle_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _data_size_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _buffer_type_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _api_buffer_size_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _pio_threshold_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _timeout_dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}


/*****************************************************************************
*
*	Function:	_dialog_write
*
*	Purpose:
*
*		Write the parameters held in memory to the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _dialog_write(HWND dlg, io_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	stat	= _mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_ch_sel_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_priority_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dma_control_mode_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _single_cycle_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _data_size_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _buffer_type_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _api_buffer_size_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _pio_threshold_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _timeout_dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	return(status);
}



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the state of the window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		msg		A status message we're to report.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, device_t* device, const char* msg)
{
	io_data_t*		data;
	unsigned int	flags;
	HMENU			menu	= GetMenu(dlg);
	U32				stat;
	U32				status	= GSC_SUCCESS;
	char			title[64];

	data	= (io_data_t*) GET_WINDOW_USER_DATA(dlg);

	data->changes	= device->handle ? data->changes : FALSE;
	strcpy(title, data->title);
	strcat(title, " I/O");
	strcat(title, data->changes ? " *" : "");
	set_win_title(dlg, data->device, FALSE, TRUE, title);

	if (data->changes)
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Cancel");
		SetDlgItemText(dlg, IDCANCEL, "Cancel");
	}
	else
	{
		ModifyMenu(menu, IDCANCEL, MF_BYCOMMAND | MF_STRING, IDCANCEL, "&Close");
		SetDlgItemText(dlg, IDCANCEL, "Close");
	}

	if (device->handle)
	{
		if ((data->was_open == FALSE) || (data->changes == FALSE))
		{
			// Update everything from the device.
			stat	= _device_read(data);
			status	= (status == GSC_SUCCESS) ? stat : status;
			stat	= _dialog_write(dlg, data);
			status	= (status == GSC_SUCCESS) ? stat : status;
		}

		data->was_open	= TRUE;
		flags			= CTRL_ENABLE;
	}
	else
	{
		// Everything is essentially disabled.
		data->changes	= FALSE;
		data->was_open	= FALSE;
		stat	= _dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		flags	= CTRL_DISABLE;
		ctrl_menu_set(dlg, IDCANCEL, CTRL_FOCUS);
	}

	ctrl_menu_set(dlg, IDOK, flags);
	ctrl_menu_set(dlg, IDC_APPLY, flags);
	ctrl_menu_set(dlg, IDC_DEFAULT, flags);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_apply
*
*	Purpose:
*
*		Respond to an APPLY request.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_apply(HWND dlg, io_data_t* data)
{
	U32	stat;
	U32	status	= GSC_SUCCESS;

	data->changes	= FALSE;

	stat	= _dialog_read(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_write(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _device_read(data);
	status	= (status == GSC_SUCCESS) ? stat : status;
	stat	= _dialog_write(dlg, data);
	status	= (status == GSC_SUCCESS) ? stat : status;

	main_refresh(data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_copy
*
*	Purpose:
*
*		Implement a COPY function for the dialog content.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_copy(HWND dlg)
{
	static const copy_t	list[]	=
	{
		/* id									flags						*/
		{ -1,									COPY_EOL | COPY_UNDERLINE	},

		#define	_FLAGS_		(COPY_INDENT | COPY_BM_GETCHECK | COPY_WIDTH | 11)

		{ IDC_IO_MODE,							COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_MODE_DMDMA,					_FLAGS_						},
		{ IDC_IO_MODE_DMA,						_FLAGS_						},
		{ IDC_IO_MODE_PIO,						COPY_EOL | _FLAGS_			},

		{ IDC_IO_DMA_CHANNEL_SELECT_MODE,		COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_DMA_CHANNEL_SELECT_STATIC,		_FLAGS_						},
		{ IDC_IO_DMA_CHANNEL_SELECT_DYNAMIC,	COPY_EOL | _FLAGS_			},

		{ IDC_IO_DMA_PRIORITY,					COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_DMA_PRIORITY_ENABLE,			_FLAGS_						},
		{ IDC_IO_DMA_PRIORITY_DISABLE,			COPY_EOL | _FLAGS_			},

		{ IDC_IO_DMA_CONTROL_MODE,				COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_DMA_CONTROL_AUTO,				_FLAGS_						},
		{ IDC_IO_DMA_CONTROL_MANUAL,			COPY_EOL | _FLAGS_			},

		{ IDC_IO_SINGLE_CYCLE,					COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_SINGLE_CYCLE_ABSENT,			_FLAGS_						},
		{ IDC_IO_SINGLE_CYCLE_PRESENT,			COPY_EOL | _FLAGS_			},

		{ IDC_IO_DATA_SIZE,						COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_DATA_SIZE_32,					_FLAGS_						},
		{ IDC_IO_DATA_SIZE_16,					_FLAGS_						},
		{ IDC_IO_DATA_SIZE_8,					COPY_EOL | _FLAGS_			},

		{ IDC_IO_XFER_BUFFER,					COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_XFER_BUFFER_APP,				_FLAGS_						},
		{ IDC_IO_XFER_BUFFER_API,				COPY_EOL | _FLAGS_			},

		{ IDC_IO_API_BUFFER_SIZE,				COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_API_BUFFER_SIZE_EDIT,			COPY_INDENT					},
		{ IDC_IO_API_BUFFER_SIZE_TEXT,			COPY_EOL | COPY_INDENT		},

		{ IDC_IO_PIO_THRESHOLD,					COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_PIO_THRESHOLD_EDIT,			COPY_INDENT					},
		{ IDC_IO_PIO_THRESHOLD_TEXT,			COPY_EOL | COPY_INDENT		},

		{ IDC_IO_TIMEOUT,						COPY_EOL | COPY_DISABLED	},
		{ IDC_IO_TIMEOUT_EDIT,					COPY_INDENT					},
		{ IDC_IO_TIMEOUT_TEXT,					COPY_EOL | COPY_INDENT		},

		{ 0,			COPY_UNDERLINE										},
		{ 0,			COPY_EOL											},

		{ IDOK,			COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDCANCEL,		COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDC_APPLY,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			},
		{ IDC_DEFAULT,	COPY_DISABLED | COPY_EOL | COPY_WIDTH | 8			}
	};

	copy_list_to_clipboard(dlg, list, ARRAY_ELEMENTS(list));
}



/*****************************************************************************
*
*	Function:	_idc_default
*
*	Purpose:
*
*		Respond to a DEFAULT request.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_default(HWND dlg, io_data_t* data)
{
	U32	status;

	data->changes	= TRUE;

	data->mode.setting			= data->mode.defalt;
	data->dma_ch_sel.setting	= data->dma_ch_sel.defalt;
	data->dma_pri.setting		= data->dma_pri.defalt;
	data->dma_ctrl_mode.setting	= data->dma_ctrl_mode.defalt;
	data->data_size.setting		= data->data_size.defalt;
	data->app_buf.setting		= data->app_buf.defalt;
	data->api_buf_size.setting	= data->api_buf_size.defalt;
	data->pio_threshold.setting	= data->pio_threshold.defalt;
	data->single_cycle.setting	= data->single_cycle.defalt;
	data->timeout.setting		= data->timeout.defalt;

	status	= _dialog_write(dlg, data);

	_refresh_window(dlg, data->device, NULL);
	error_status(dlg, 0, status);
}



/*****************************************************************************
*
*	Function:	_idc_radio_button
*
*	Purpose:
*
*		Respond to activity on a group of radio buttons.
*
*	Arguments:
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*		item	The structure for the group of radio buttons accessed.
*
*		changed	Tell the caller if there was a change.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_radio_button(
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data,
	_io_t*		item,
	int*		changed)
{
	int		code;
	HWND	control;
	LRESULT	ret;
	LRESULT	state;

	if (item->ignore)
	{
		ret			= 1;
		changed[0]	= FALSE;
	}
	else
	{
		code	= HIWORD(wParam);
		control	= (HWND) lParam;

		if (code == BN_SETFOCUS)
		{
			state		= SendMessage(control, BM_GETCHECK, 0, 0);
			ret			= 1;
			changed[0]	= FALSE;

			if (state == BST_UNCHECKED)
				item->control	= control;
			else
				item->control	= NULL;
		}
		else if ((code == BN_CLICKED) && (item->control == control))
		{
			// We get here only if a change has occurred.
			data->changes	= TRUE;
			ret				= 0;
			changed[0]		= TRUE;
		}
		else
		{
			ret			= 1;
			changed[0]	= FALSE;
		}
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_mode
*
*	Purpose:
*
*		Respond to activity on the I/O Mode radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_mode(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;
	U32		stat;
	U32		status;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->mode,
							&changed);

	if (changed)
	{
		status	= GSC_SUCCESS;

		stat	= _mode_dialog_read(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _mode_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _dma_ch_sel_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _dma_priority_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _dma_control_mode_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _single_cycle_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _pio_threshold_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		_refresh_window(dlg, data->device, NULL);
		error_status(dlg, 0, status);
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_dma_channel_select
*
*	Purpose:
*
*		Respond to activity on the I/O DMA Channel Select radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_dma_channel_select(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->dma_ch_sel,
							&changed);

	if (changed)
		_refresh_window(dlg, data->device, NULL);

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_dma_priority
*
*	Purpose:
*
*		Respond to activity on the I/O DMA Priority radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_dma_priority(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->dma_pri,
							&changed);

	if (changed)
		_refresh_window(dlg, data->device, NULL);

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_dma_control_mode
*
*	Purpose:
*
*		Respond to activity on the I/O DMA Control Mode radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_dma_control_mode(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->dma_ctrl_mode,
							&changed);

	if (changed)
		_refresh_window(dlg, data->device, NULL);

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_data_size
*
*	Purpose:
*
*		Respond to activity on the I/O Data Size radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_data_size(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;
	U32		stat;
	U32		status;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->data_size,
							&changed);

	if (changed)
	{
		status	= _data_size_dialog_read(dlg, data);
		stat	= _single_cycle_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		_refresh_window(dlg, data->device, NULL);
		error_status(dlg, 0, status);
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_xfer_buffer
*
*	Purpose:
*
*		Respond to activity on the I/O Transfer buffer Type radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_xfer_buffer(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;
	U32		stat;
	U32		status;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->app_buf,
							&changed);

	if (changed)
	{
		status	= GSC_SUCCESS;

		stat	= _buffer_type_dialog_read(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _api_buffer_size_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		_refresh_window(dlg, data->device, NULL);
		error_status(dlg, 0, status);
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_api_buffer_size_edit
*
*	Purpose:
*
*		Respond to activity in the API Buffer Size edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_io_api_buffer_size_edit(HWND dlg, WPARAM wParam, io_data_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->api_buf_size.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idc_io_pio_threshold_edit
*
*	Purpose:
*
*		Respond to activity in the I/O PIO Threshold edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_io_pio_threshold_edit(HWND dlg, WPARAM wParam, io_data_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->pio_threshold.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idc_io_single_cycle
*
*	Purpose:
*
*		Respond to activity on the I/O Single Cycle radio buttons.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*		data	The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static LRESULT _idc_io_single_cycle(
	HWND		dlg,
	WPARAM		wParam,
	LPARAM		lParam,
	io_data_t*	data)
{
	int		changed;
	LRESULT	ret;
	U32		stat;
	U32		status;

	ret	= _idc_radio_button(wParam,
							lParam,
							data,
							&data->app_buf,
							&changed);

	if (changed)
	{
		status	= GSC_SUCCESS;

		stat	= _single_cycle_dialog_read(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;
		stat	= _single_cycle_dialog_write(dlg, data);
		status	= (status == GSC_SUCCESS) ? stat : status;

		_refresh_window(dlg, data->device, NULL);
		error_status(dlg, 0, status);
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_idc_io_timeout_edit
*
*	Purpose:
*
*		Respond to activity in the I/O Timeout edit control.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		wParam		The message's WORD sized parameter.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idc_io_timeout_edit(HWND dlg, WPARAM wParam, io_data_t* data)
{
	int	code	= HIWORD(wParam);

	if ((code == EN_CHANGE) && (data->timeout.ignore == FALSE))
	{
		data->changes	= TRUE;
		_refresh_window(dlg, data->device, NULL);
	}
}



/*****************************************************************************
*
*	Function:	_idcancel
*
*	Purpose:
*
*		Respond to a request to close the dialog.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idcancel(HWND dlg)
{
	io_data_t*	data;

	main_refresh_add(dlg, NULL);
	data	= (io_data_t*) GET_WINDOW_USER_DATA(dlg);

	if (data)
	{
		data->dlg	= NULL;
		win_pos_save(dlg, data->section_name);
	}

	DestroyWindow(dlg);
}



/*****************************************************************************
*
*	Function:	_idok
*
*	Purpose:
*
*		Process clicking of the OK button.
*
*	Arguments:
*
*		dlg			The dalog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _idok(HWND dlg, io_data_t* data)
{
	_idc_apply(dlg, data);
	_idcancel(dlg);
}



/*****************************************************************************
*
*	Function:	_wm_command
*
*	Purpose:
*
*		Process WM_COMMAND messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's WORD parameter.
*
*		lParam		The message's LONG parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_command(HWND dlg, WPARAM wParam, LPARAM lParam)
{
	io_data_t*	data	= (io_data_t*) GET_WINDOW_USER_DATA(dlg);
	int			id		= LOWORD(wParam);
	LRESULT		ret		= TRUE;

	switch (id)
	{
		default:

			ret	= FALSE;
			break;

		case IDC_APPLY:

			_idc_apply(dlg, data);
			break;

		case IDC_IO_API_BUFFER_SIZE_EDIT:

			_idc_io_api_buffer_size_edit(dlg, wParam, data);
			break;

		case IDC_COPY:

			_idc_copy(dlg);
			break;

		case IDC_DEFAULT:

			_idc_default(dlg, data);
			break;

		case IDC_IO_DATA_SIZE_8:
		case IDC_IO_DATA_SIZE_16:
		case IDC_IO_DATA_SIZE_32:

			_idc_io_data_size(dlg, wParam, lParam, data);
			break;

		case IDC_IO_DMA_CHANNEL_SELECT_DYNAMIC:
		case IDC_IO_DMA_CHANNEL_SELECT_STATIC:

			_idc_io_dma_channel_select(dlg, wParam, lParam, data);
			break;

		case IDC_IO_DMA_CONTROL_AUTO:
		case IDC_IO_DMA_CONTROL_MANUAL:

			_idc_io_dma_control_mode(dlg, wParam, lParam, data);
			break;

		case IDC_IO_DMA_PRIORITY_DISABLE:
		case IDC_IO_DMA_PRIORITY_ENABLE:

			_idc_io_dma_priority(dlg, wParam, lParam, data);
			break;

		case IDC_IO_MODE_DMA:
		case IDC_IO_MODE_DMDMA:
		case IDC_IO_MODE_PIO:

			_idc_io_mode(dlg, wParam, lParam, data);
			break;

		case IDC_IO_PIO_THRESHOLD_EDIT:

			_idc_io_pio_threshold_edit(dlg, wParam, data);
			break;

		case IDC_IO_SINGLE_CYCLE_ABSENT:
		case IDC_IO_SINGLE_CYCLE_PRESENT:

			_idc_io_single_cycle(dlg, wParam, lParam, data);
			break;

		case IDC_IO_TIMEOUT_EDIT:

			_idc_io_timeout_edit(dlg, wParam, data);
			break;

		case IDC_IO_XFER_BUFFER_API:
		case IDC_IO_XFER_BUFFER_APP:

			_idc_io_xfer_buffer(dlg, wParam, lParam, data);
			break;

		case IDCANCEL:

			_idcancel(dlg);
			break;

		case IDOK:

			_idok(dlg, data);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_wm_destroy
*
*	Purpose:
*
*		Process WM_DESTROY messages.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*	Returned:
*
*		0			We processed the message.
*
*****************************************************************************/

static LRESULT _wm_destroy(HWND dlg)
{
	_idcancel(dlg);
	DestroyWindow(dlg);
	return(0);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		lParam		The message's long sized argument.
*
*	Returned:
*
*		TRUE		Windows should set the initial control focus.
*		FALSE		Windows should NOT set the initial control focus.
*
*****************************************************************************/

static LRESULT _wm_initdialog(HWND dlg, LPARAM lParam)
{
	char		buf[32];
	HWND		control;
	io_data_t*	data;

	SET_WINDOW_USER_DATA(dlg, lParam);
	data	= (io_data_t*) GET_WINDOW_USER_DATA(dlg);

	SendMessage(dlg, WM_SETICON, ICON_SMALL, (LPARAM) data->device->icon.little);
	SendMessage(dlg, WM_SETICON, ICON_BIG, (LPARAM) data->device->icon.big);

	control	= GetDlgItem(dlg, IDC_IO_API_BUFFER_SIZE_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "Bytes (1K = %d)", _buffer_size_value_range.k);
	ctrl_menu_set_text(dlg, IDC_IO_API_BUFFER_SIZE_TEXT, buf);

	control	= GetDlgItem(dlg, IDC_IO_PIO_THRESHOLD_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "Samples (1K = %d)", _pio_threshold_value_range.k);
	ctrl_menu_set_text(dlg, IDC_IO_PIO_THRESHOLD_TEXT, buf);

	control	= GetDlgItem(dlg, IDC_IO_TIMEOUT_EDIT);
	SendMessage(control, EM_LIMITTEXT, 12, 0);

	sprintf(buf, "Seconds (1K = %d)", _timeout_value_range.k);
	ctrl_menu_set_text(dlg, IDC_IO_TIMEOUT_TEXT, buf);

	win_pos_restore(dlg, data->section_name, FALSE);
	main_refresh_add(dlg, _refresh_window);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_wm_syscommand
*
*	Purpose:
*
*		Process a WM_SYSCOMMAND message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		wParam		The message's word sized parameter.
*
*	Returned:
*
*		TRUE		We processed the message.
*		FALSE		We did not process the message.
*
*****************************************************************************/

static LRESULT _wm_syscommand(HWND dlg, WPARAM wParam)
{
	LRESULT	ret;

	switch (wParam)
	{
		default:

			ret	= FALSE;
			break;

		case SC_CLOSE:

			_idcancel(dlg);
			ret	= TRUE;
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dlg_proc
*
*	Purpose:
*
*		Processes messages for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		msg			The message to process.
*
*		wParam		The message's word sized parameter.
*
*		lParam		The message's long sized parameter.
*
*	Returned:
*
*		A message specific value.
*
*****************************************************************************/

static INT_PTR CALLBACK _dlg_proc(	HWND	dlg,
									UINT	msg,
									WPARAM	wParam,
									LPARAM	lParam)
{
	LRESULT	ret;

	switch (msg)
	{
		default:

			ret	= FALSE;
			break;

		case WM_COMMAND:

			ret	= _wm_command(dlg, wParam, lParam);
			break;

		case WM_DESTROY:

			ret	= _wm_destroy(dlg);
			break;

		case WM_INITDIALOG:

			ret	= _wm_initdialog(dlg, lParam);
			break;

		case WM_SYSCOMMAND:

			ret	= _wm_syscommand(dlg, wParam);
			break;
	}

	return(ret);
}



/*****************************************************************************
*
*	Function:	_dma_ch_sel_cfg_wiz
*
*	Purpose:
*
*		Provide customized processing for the Configuration Wizards processing
*		of the DMA Channel Select configuration option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device structure to access.
*
*		entry		The structure for the parameter to process.
*
*		value		The current argument value.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status;
*
*****************************************************************************/

static U32 _dma_ch_sel_cfg_wiz(	HWND				dlg,
								device_t*			device,
								const parm_entry_t*	entry,
								U32					value)
{
	char	buf[4096];
	HWND	control;
	U32		feature;
	int		id;
	char*	psz;
	U32		status;

	status	= HPDI32_MISC_FEATURES__DMA_CH1(device->handle, &feature);

	if (feature == HPDI32_MISC_FEATURES_PRESENT)
	{
		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		strcat(	buf,
				" This HPDI32 does support Demand Mode DMA on DMA Channel 1.");
		SetWindowText(control, buf);

		if (_dma_ch_sel_value_list[0].value == HPDI32_IO_DMA_CHANNEL_SEL_STATIC)
			id	= IDC_CFG_WIZ_DATA_RADIO_1;
		else
			id	= IDC_CFG_WIZ_DATA_RADIO_2;

		control	= GetDlgItem(dlg, id);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		buf[strlen(buf) - 1]	= 0;
		strcat(buf, " This is the preferred option.)");
		SetWindowText(control, buf);
	}
	else
	{
		control	= GetDlgItem(dlg, IDC_CFG_WIZ_DATA_EDIT);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		strcat(	buf,
				" This HPDI32 does not support"
				" Demand Mode DMA on DMA Channel 1.");
		SetWindowText(control, buf);

		if (_dma_ch_sel_value_list[0].value == HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC)
			id	= IDC_CFG_WIZ_DATA_RADIO_1;
		else
			id	= IDC_CFG_WIZ_DATA_RADIO_2;

		control	= GetDlgItem(dlg, id);
		SendMessage(control, WM_GETTEXT, (WPARAM) sizeof(buf), (LPARAM) buf);
		psz	= strchr(buf, '(');

		if (psz)
			psz[0]	= 0;

		strcat(buf, " (Consider this option for this board.)");
		SetWindowText(control, buf);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_service_device_close
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_DEVICE_CLOSE service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_device_close(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	io_data_t*	data	= (io_data_t*) wParam;

	if (data->which & HPDI32_WHICH_TX)
	{
		ini_bool_set(NULL, data->section_name, KEY_APP_BUFFER, device->tx.app_buf);
		ini_u32_set(NULL, data->section_name, KEY_API_BUFFER_SIZE, device->tx.buf_size);
	}
	else
	{
		ini_bool_set(NULL, data->section_name, KEY_APP_BUFFER, device->rx.app_buf);
		ini_u32_set(NULL, data->section_name, KEY_API_BUFFER_SIZE, device->rx.buf_size);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_device_open
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_DEVICE_OPEN service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_device_open(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	io_data_t*	data	= (io_data_t*) wParam;
	U32			defalt;
	U32			size;

	defalt	= PREF_API_BUFFER_SIZE__GET();
	ini_u32_get(NULL, data->section_name, KEY_API_BUFFER_SIZE, &size, defalt);
	data->api_buf_size.defalt	= defalt;

	if (data->which & HPDI32_WHICH_TX)
	{
		device->tx.app_buf	= TRUE;
		ini_u32_get(NULL,
					data->section_name,
					KEY_API_BUFFER_SIZE,
					&device->tx.buf_size,
					defalt);
	}
	else
	{
		device->rx.app_buf	= TRUE;
		ini_u32_get(NULL,
					data->section_name,
					KEY_API_BUFFER_SIZE,
					&device->rx.buf_size,
					defalt);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	io_data_t*	data	= (io_data_t*) wParam;

	if (device->handle)
		ctrl_menu_set(dlg, data->id_button, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, data->id_button, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	io_data_t*	data	= (io_data_t*) wParam;

	data->device	= device;

	if (data->dlg)
	{
		SetFocus(data->dlg);
	}
	else
	{
		data->changes				= FALSE;
		data->api_buf_size.ignore	= TRUE;
		data->pio_threshold.ignore	= TRUE;
		data->timeout.ignore		= TRUE;
		data->was_open				= FALSE;
		data->dlg					= CreateDialogParam(data->device->instance,
														(LPCTSTR) IDD_IO,
														dlg,
														(DLGPROC) _dlg_proc,
														(LPARAM) data);
		ShowWindow(data->dlg, SW_SHOWNORMAL);
		data->api_buf_size.ignore	= FALSE;
		data->pio_threshold.ignore	= FALSE;
		data->timeout.ignore		= FALSE;
		_refresh_window(data->dlg, data->device, NULL);
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	io_service
*
*	Purpose:
*
*		Implement the common service entry point for the I/O services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT io_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function				*/
		{ SERVICE_DEVICE_CLOSE,	_service_device_close	},
		{ SERVICE_DEVICE_OPEN,	_service_device_open	},
		{ SERVICE_REFRESH,		_service_refresh		},
		{ SERVICE_WM_COMMAND,	_service_wm_command		},
		{ SERVICE_END_LIST,		NULL					}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


