// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/regs_fw_bcr.c $
// $Rev: 2809 $
// $Date: 2009-07-29 21:26:35 -0500 (Wed, 29 Jul 2009) $

/*****************************************************************************
*
*	Provide data describing the Board Control Register.
*
*****************************************************************************/

#include "regs.h"



/* variables	*************************************************************/

static const reg_value_t	_br_values[]	=
{
	{
	/* name		*/	"Don't Reset",
	/* value	*/	0
	},
	{
	/* name		*/	"Reset",
	/* value	*/	HPDI32_BCR_BOARD_RESET
	}
};

static const reg_value_t	_c0_values[]	=
{
	{
	/* name		*/	"Frame Valid",
	/* value	*/	HPDI32_BCR_C0_FV
	},
	{
	/* name		*/	"GPIO 6 Input",
	/* value	*/	HPDI32_BCR_C0_IN
	},
	{
	/* name		*/	"GPIO 6 Output Hi",
	/* value	*/	HPDI32_BCR_C0_OUT_H
	},
	{
	/* name		*/	"GPIO 6 Output Low",
	/* value	*/	HPDI32_BCR_C0_OUT_L
	}
};

static const reg_value_t	_c1_values[]	=
{
	{
	/* name		*/	"Line Valid",
	/* value	*/	HPDI32_BCR_C1_LV
	},
	{
	/* name		*/	"GPIO 0 Input",
	/* value	*/	HPDI32_BCR_C1_IN
	},
	{
	/* name		*/	"GPIO 0 Output Hi",
	/* value	*/	HPDI32_BCR_C1_OUT_H
	},
	{
	/* name		*/	"GPIO 0 Output Low",
	/* value	*/	HPDI32_BCR_C1_OUT_L
	}
};

static const reg_value_t	_c2_values[]	=
{
	{
	/* name		*/	"Status Valid",
	/* value	*/	HPDI32_BCR_C2_SV
	},
	{
	/* name		*/	"GPIO 1 Input",
	/* value	*/	HPDI32_BCR_C2_IN
	},
	{
	/* name		*/	"GPIO 1 Output Hi",
	/* value	*/	HPDI32_BCR_C2_OUT_H
	},
	{
	/* name		*/	"GPIO 1 Output Low",
	/* value	*/	HPDI32_BCR_C2_OUT_L
	}
};

static const reg_value_t	_c3_values[]	=
{
	{
	/* name		*/	"Rx Ready",
	/* value	*/	HPDI32_BCR_C3_RR
	},
	{
	/* name		*/	"GPIO 2 Input",
	/* value	*/	HPDI32_BCR_C3_IN
	},
	{
	/* name		*/	"GPIO 2 Output Hi",
	/* value	*/	HPDI32_BCR_C3_OUT_H
	},
	{
	/* name		*/	"GPIO 2 Output Low",
	/* value	*/	HPDI32_BCR_C3_OUT_L
	}
};

static const reg_value_t	_c4_values[]	=
{
	{
	/* name		*/	"Tx Data Ready",
	/* value	*/	HPDI32_BCR_C4_TR
	},
	{
	/* name		*/	"GPIO 3 Input",
	/* value	*/	HPDI32_BCR_C4_IN
	},
	{
	/* name		*/	"GPIO 3 Output Hi",
	/* value	*/	HPDI32_BCR_C4_OUT_H
	},
	{
	/* name		*/	"GPIO 3 Output Low",
	/* value	*/	HPDI32_BCR_C4_OUT_L
	}
};

static const reg_value_t	_c5_values[]	=
{
	{
	/* name		*/	"Tx Enabled",
	/* value	*/	HPDI32_BCR_C5_TE
	},
	{
	/* name		*/	"GPIO 4 Input",
	/* value	*/	HPDI32_BCR_C5_IN
	},
	{
	/* name		*/	"GPIO 4 Output Hi",
	/* value	*/	HPDI32_BCR_C5_OUT_H
	},
	{
	/* name		*/	"GPIO 4 Output Low",
	/* value	*/	HPDI32_BCR_C5_OUT_L
	}
};

static const reg_value_t	_c6_values[]	=
{
	{
	/* name		*/	"Rx Enabled",
	/* value	*/	HPDI32_BCR_C6_RE
	},
	{
	/* name		*/	"GPIO 5 Input",
	/* value	*/	HPDI32_BCR_C6_IN
	},
	{
	/* name		*/	"GPIO 5 Output Hi",
	/* value	*/	HPDI32_BCR_C6_OUT_H
	},
	{
	/* name		*/	"GPIO 5 Output Low",
	/* value	*/	HPDI32_BCR_C6_OUT_L
	}
};

static const reg_value_t	_cte_values[]	=
{
	{
	/* name		*/	"Disable/Local",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable/Remote",
	/* value	*/	HPDI32_BCR_TX_THROTTLE
	}
};

static const reg_value_t	_dmdma0d_values[]	=
{
	{
	/* name		*/	"Receive",
	/* value	*/	0
	},
	{
	/* name		*/	"Transmit",
	/* value	*/	HPDI32_BCR_DMDMA0_DIR
	}
};

static const reg_value_t	_lvhsvh_values[]	=
{
	{
	/* name		*/	"Disable/Stay Low",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable/Go High",
	/* value	*/	HPDI32_BCR_STATUS_VALID
	}
};

static const reg_value_t	_re_values[]	=
{
	{
	/* name		*/	"Disable",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable",
	/* value	*/	HPDI32_BCR_RX_ENABLED
	}
};

static const reg_value_t	_rfr_values[]	=
{
	{
	/* name		*/	"Don't Reset",
	/* value	*/	0
	},
	{
	/* name		*/	"Reset",
	/* value	*/	HPDI32_BCR_RX_FIFO_RESET
	}
};

static const reg_value_t	_scd_values[]	=
{
	{
	/* name		*/	"Single Cycles Enabled",
	/* value	*/	0
	},
	{
	/* name		*/	"Single Cycles Disabled",
	/* value	*/	HPDI32_BCR_SING_CYC_DIS
	}
};

static const reg_value_t	_te_values[]	=
{
	{
	/* name		*/	"Disable",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable",
	/* value	*/	HPDI32_BCR_TX_ENABLED
	}
};

static const reg_value_t	_tfr_values[]	=
{
	{
	/* name		*/	"Don't Reset",
	/* value	*/	0
	},
	{
	/* name		*/	"Reset",
	/* value	*/	HPDI32_BCR_TX_FIFO_RESET
	}
};

static const reg_value_t	_tm_values[]	=
{
	{
	/* name		*/	"Disable",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable",
	/* value	*/	HPDI32_BCR_TEST_MODE
	}
};

static const reg_value_t	_ts_values[]	=
{
	{
	/* name		*/	"Disable/Stop",
	/* value	*/	0
	},
	{
	/* name		*/	"Enable/Start",
	/* value	*/	HPDI32_BCR_TX_START
	}
};

static const reg_value_t	_tsfed_values[]	=
{
	{
	/* name		*/	"Auto Clear",
	/* value	*/	0
	},
	{
	/* name		*/	"Stay Set",
	/* value	*/	HPDI32_BCR_TX_AUTO_STOP_NO
	}
};

static reg_field_t	_fields[]	=
{
	{
	/* name		*/	"TM",
	/* desc		*/	"Text Mode",
	/* mask		*/	HPDI32_BCR_TEST_MODE,
	/* value	*/	_tm_values,
	/* values	*/	ARRAY_ELEMENTS(_tm_values)
	},
	{
	/* name		*/	"C6",
	/* desc		*/	"Cable Command 6",
	/* mask		*/	HPDI32_BCR_C6_MASK,
	/* value	*/	_c6_values,
	/* values	*/	ARRAY_ELEMENTS(_c6_values)
	},
	{
	/* name		*/	"C5",
	/* desc		*/	"Cable Command 5",
	/* mask		*/	HPDI32_BCR_C5_MASK,
	/* value	*/	_c5_values,
	/* values	*/	ARRAY_ELEMENTS(_c5_values)
	},
	{
	/* name		*/	"C4",
	/* desc		*/	"Cable Command 4",
	/* mask		*/	HPDI32_BCR_C4_MASK,
	/* value	*/	_c4_values,
	/* values	*/	ARRAY_ELEMENTS(_c4_values)
	},
	{
	/* name		*/	"C3",
	/* desc		*/	"Cable Command 3",
	/* mask		*/	HPDI32_BCR_C3_MASK,
	/* value	*/	_c3_values,
	/* values	*/	ARRAY_ELEMENTS(_c3_values)
	},
	{
	/* name		*/	"C2",
	/* desc		*/	"Cable Command 2",
	/* mask		*/	HPDI32_BCR_C2_MASK,
	/* value	*/	_c2_values,
	/* values	*/	ARRAY_ELEMENTS(_c2_values)
	},
	{
	/* name		*/	"C1",
	/* desc		*/	"Cable Command 1",
	/* mask		*/	HPDI32_BCR_C1_MASK,
	/* value	*/	_c1_values,
	/* values	*/	ARRAY_ELEMENTS(_c1_values)
	},
	{
	/* name		*/	"C0",
	/* desc		*/	"Cable Command 0",
	/* mask		*/	HPDI32_BCR_C0_MASK,
	/* value	*/	_c0_values,
	/* values	*/	ARRAY_ELEMENTS(_c0_values)
	},
	{
	/* name		*/	"RES",
	/* desc		*/	"Reserved",
	/* mask		*/	HPDI32_BCR_RESERVED_3,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"RES",
	/* desc		*/	"Reserved",
	/* mask		*/	HPDI32_BCR_RESERVED_2,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"SCD",
	/* desc		*/	"Single Cycle Disable",
	/* mask		*/	HPDI32_BCR_SING_CYC_DIS,
	/* value	*/	_scd_values,
	/* values	*/	ARRAY_ELEMENTS(_scd_values)
	},
	{
	/* name		*/	"DMDMA0D",
	/* desc		*/	"Demand Mode DMA Channel 0 Direction",
	/* mask		*/	HPDI32_BCR_DMDMA0_DIR,
	/* value	*/	_dmdma0d_values,
	/* values	*/	ARRAY_ELEMENTS(_dmdma0d_values)
	},
	{
	/* name		*/	"CTE",
	/* desc		*/	"Cable Throttle Enable",
	/* mask		*/	HPDI32_BCR_TX_THROTTLE,
	/* value	*/	_cte_values,
	/* values	*/	ARRAY_ELEMENTS(_cte_values)
	},
	{
	/* name		*/	"TS",
	/* desc		*/	"Tx Start",
	/* mask		*/	HPDI32_BCR_TX_START,
	/* value	*/	_ts_values,
	/* values	*/	ARRAY_ELEMENTS(_ts_values)
	},
	{
	/* name		*/	"LVHSVH",
	/* desc		*/	"Line Valid Hi on Status Valid Hi",
	/* mask		*/	HPDI32_BCR_STATUS_VALID,
	/* value	*/	_lvhsvh_values,
	/* values	*/	ARRAY_ELEMENTS(_lvhsvh_values)
	},
	{
	/* name		*/	"TSFED",
	/* desc		*/	"Tx Stop on FIFO Empty Disable",
	/* mask		*/	HPDI32_BCR_TX_AUTO_STOP_NO,
	/* value	*/	_tsfed_values,
	/* values	*/	ARRAY_ELEMENTS(_tsfed_values)
	},
	{
	/* name		*/	"RE",
	/* desc		*/	"Rx Enable",
	/* mask		*/	HPDI32_BCR_RX_ENABLED,
	/* value	*/	_re_values,
	/* values	*/	ARRAY_ELEMENTS(_re_values)
	},
	{
	/* name		*/	"TE",
	/* desc		*/	"Tx Enable",
	/* mask		*/	HPDI32_BCR_TX_ENABLED,
	/* value	*/	_te_values,
	/* values	*/	ARRAY_ELEMENTS(_te_values)
	},
	{
	/* name		*/	"RES",
	/* desc		*/	"Reserved",
	/* mask		*/	HPDI32_BCR_RESERVED_1,
	/* value	*/	NULL,
	/* values	*/	0
	},
	{
	/* name		*/	"RFR",
	/* desc		*/	"Rx FIFO Reset",
	/* mask		*/	HPDI32_BCR_RX_FIFO_RESET,
	/* value	*/	_rfr_values,
	/* values	*/	ARRAY_ELEMENTS(_rfr_values)
	},
	{
	/* name		*/	"TFR",
	/* desc		*/	"Tx FIFO Reset",
	/* mask		*/	HPDI32_BCR_TX_FIFO_RESET,
	/* value	*/	_tfr_values,
	/* values	*/	ARRAY_ELEMENTS(_tfr_values)
	},
	{
	/* name		*/	"BR",
	/* desc		*/	"Board Reset",
	/* mask		*/	HPDI32_BCR_BOARD_RESET,
	/* value	*/	_br_values,
	/* values	*/	ARRAY_ELEMENTS(_br_values)
	}
};

reg_def_t	reg_fw_bcr	=
{
	/* name			*/	"BCR",
	/* desc			*/	"Board Control Register",
	/* reg_id		*/	HPDI32_BCR,
	/* read_only	*/	HPDI32_BCR_RESERVED_1 |
						HPDI32_BCR_RESERVED_2 |
						HPDI32_BCR_RESERVED_3,
	/* field		*/	_fields,
	/* fields		*/	ARRAY_ELEMENTS(_fields),
	/* init			*/	NULL,
	/* refresh		*/	NULL
};


