// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_clock.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Rx Clock configuration options.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "resource.h"
#include "static.h"
#include "tx.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Rx Clock control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	static const wave_t	list[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	11,
		/* w1		*/	2,
		/* w2		*/	2
		},
		{ 0 }
	};

	int	x;
	int	y;

	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	x	+= static_text_paint(control, data, x, y, " ");
	static_wave_paint(data, x, y, list);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		// Rx Clock
		{
		/* id		*/	IDC_RX_CLOCK,
		/* ref		*/	IDC_RX_ENABLED,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_CLOCK,
		/* ref		*/	IDC_RX_ENABLED,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_CLOCK_ARROW,
		/* ref		*/	IDC_RX_ENABLED_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_CLOCK_ARROW,
		/* ref		*/	IDC_RX_ENABLED,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_RX_CLOCK);
	static_arrow_direction_set(dlg, IDC_RX_CLOCK, -1);
	static_paint_function_set(dlg, IDC_RX_CLOCK, _gui_paint);
	arrow_init(dlg, device, IDC_RX_CLOCK_ARROW);
	arrow_type_set(dlg, IDC_RX_CLOCK_ARROW, ARROW_LEFT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[16];
	U32		cd;
	U32		clock;
	int		len;
	char*	psz;
	U32		rx;
	U32		status;
	U32		tx;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_RX_CLOCK, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_RX_CLOCK_ARROW, CTRL_DISABLE);
			break;
		}

		// The device is open.
		status	= HPDI32_RX_ENABLE__GET(device->handle, &rx);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_TX_ENABLE__GET(device->handle, &tx);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_TX_CLOCK_DIVIDER__GET(device->handle, &cd);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (cd < 1)
			clock	= device->oscillator;
		else
			clock	= device->oscillator / (cd * 2);

		format_u32(clock, TX_RATE_K, buf);
		len	= (int) strlen(buf);

		if (len > 5)
		{
			format_float((float) cd, buf);
			psz	= strchr(buf, ' ');

			if (psz)
			{
				psz[0]	= psz[1];
				psz[1]	= 0;
			}
		}

		strcat(buf, "Hz");

		if ((rx == HPDI32_RX_ENABLE_YES) && (tx == HPDI32_TX_ENABLE_YES))
		{
			// The signal is input to the receiver from the transmitter.
			ctrl_menu_set(dlg, IDC_RX_CLOCK, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_CLOCK_ARROW, CTRL_ENABLE);
			ctrl_menu_set_text(dlg, IDC_RX_CLOCK_ARROW, buf);
			break;
		}
		else if (rx == HPDI32_RX_ENABLE_YES)
		{
			// The signal is input to the receiver from the cable.
			ctrl_menu_set(dlg, IDC_RX_CLOCK, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_CLOCK_ARROW, CTRL_ENABLE | CTRL_NULL);
			break;
		}

		// The signal is unused by the receiver at this time.
		ctrl_menu_set(dlg, IDC_RX_CLOCK, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_RX_CLOCK_ARROW, CTRL_DISABLE | CTRL_NULL);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	rx_clock_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx Clock services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_clock_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


