// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_data.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Rx Data options.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "cable.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Rx Data control.
*
*	Arguments:
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	static const wave_t	list[]	=
	{
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	4,
		/* w1		*/	4,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	-4,
		/* count	*/	1,
		/* w1		*/	4,
		/* w2		*/	4
		},
		{
		/* flags	*/	WAVE_T_B,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	2,
		/* w2		*/	0
		},
		{ 0 }
	};

	int		dx;
	int		dy;
	HPEN	pen;
	int		x;
	int		y;

	// Left of arrow.
	x	= data->inside.rect.left
		+ data->arrow.offset
		- MARGIN;
	y	= data->inside.rect.top;

	x	-= static_text_paint(control, data, x, y, " ");

	dx	= data->arrow.width / 2;
	dy	= (data->arrow.height / 2) * 2;
	y	= (y + (data->tmAscent / 2)) - (dy / 2);
	pen	= SelectObject(data->dc, data->arrow.pen);
	y	+= dy;
	MoveToEx(data->dc, x, y, NULL);
	x	-= dx;
	LineTo(data->dc, x, y);
	y	-= dy;
	LineTo(data->dc, x, y );
	x	-= dx + 1;
	LineTo(data->dc, x, y);
	SelectObject(data->dc, pen);

	// Right of arrow.
	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	x	+= static_text_paint(control, data, x, y, " ");
	static_wave_paint(data, x, y, list);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX_DATA,
		/* ref		*/	IDC_RX_CLOCK,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_DATA,
		/* ref		*/	IDC_RX_CLOCK,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_DATA_ARROW,
		/* ref		*/	IDC_RX_CLOCK_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_DATA_ARROW,
		/* ref		*/	IDC_RX_CLOCK_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_DATA_ARROW,
		/* ref		*/	IDC_RX_DATA_ARROW,
		/* flags	*/	ALIGN_T_T,
		/* value	*/	- MARGIN
		},
		{
		/* id		*/	IDC_RX_DATA_ARROW,
		/* ref		*/	IDC_RX_DATA_ARROW,
		/* flags	*/	ALIGN_B_B,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_RX_DATA);
	static_arrow_direction_set(dlg, IDC_RX_DATA, -1);
	static_paint_function_set(dlg, IDC_RX_DATA, _gui_paint);
	arrow_init(dlg, device, IDC_RX_DATA_ARROW);
	arrow_type_set(dlg, IDC_RX_DATA_ARROW, ARROW_LEFT_2);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	char	buf[64];
	U32		cd;
	U32		clock;
	int		len;
	char*	psz;
	U32		rate;
	U32		rx;
	U32		status;
	U32		tx;
	U32		width;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_RX_DATA, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_RX_DATA_ARROW, CTRL_DISABLE | CTRL_NULL);
			break;
		}

		// The device is open.
		status	= HPDI32_IO_DATA_SIZE__RX_GET(device->handle, &width);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_TX_CLOCK_DIVIDER__GET(device->handle, &cd);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_TX_ENABLE__GET(device->handle, &tx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (cd < 1)
			clock	= device->oscillator;
		else
			clock	= device->oscillator / (cd * 2);

		switch (width)
		{
			default:
			case HPDI32_IO_DATA_SIZE_8_BITS:

				psz		= "8-bits";
				rate	= clock;
				break;

			case HPDI32_IO_DATA_SIZE_16_BITS:

				psz		= "16-bits";
				rate	= clock * 2;
				break;

			case HPDI32_IO_DATA_SIZE_32_BITS:

				psz		= "32-bits";
				rate	= clock * 4;
				break;
		}

		status	= HPDI32_RX_ENABLE__GET(device->handle, &rx);
		error_status(dlg, IDC_INFO_TEXT, status);

		if ((rx == HPDI32_RX_ENABLE_YES) && (tx == HPDI32_TX_ENABLE_YES))
		{
			// The receiver and transmitter are enabled.
			strcpy(buf, psz);
			strcat(buf, ", ");
			psz	= buf + strlen(buf);
			format_u32(rate, 1000, psz);
			len	= (int) strlen(psz);

			if (len > 4)
			{
				format_float((float) rate, psz);
				psz	= strchr(psz, ' ');

				if (psz)
				{
					psz[0]	= psz[1];
					psz[1]	= 0;
				}
			}

			strcat(buf, "B/S");
			ctrl_menu_set(dlg, IDC_RX_DATA, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set_text(dlg, IDC_RX_DATA_ARROW, buf);
			break;
		}

		if (rx == HPDI32_RX_ENABLE_YES)
		{
			// The receiver is enabled.
			ctrl_menu_set(dlg, IDC_RX_DATA, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_DATA_ARROW, CTRL_ENABLE);
			ctrl_menu_set_text(dlg, IDC_RX_DATA_ARROW, psz);
			break;
		}

		// The signals are unused by the receiver at this time.
		ctrl_menu_set(dlg, IDC_RX_DATA, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_RX_DATA_ARROW, CTRL_DISABLE);
		ctrl_menu_set_text(dlg, IDC_RX_DATA_ARROW, psz);
		break;
	}

	return(0);
}



/*****************************************************************************
*
*	Function:	rx_data_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx Data services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_data_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


