// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/rx_sv.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Rx Status Valid option.
*
*****************************************************************************/

#include <windows.h>

#include "rx.h"
#include "align.h"
#include "arrow.h"
#include "cable.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Rx Status Valid control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	static const wave_t	down[]	=
	{
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	up[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	int		x;
	int		y;

	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	x	+= static_text_paint(control, data, x, y, " Receive ");
	x	+= static_wave_paint(data, x, y, up);

	x	+= static_text_paint(control, data, x, y, "  Ignore ");
	static_wave_paint(data, x, y, down);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_RX_SV,
		/* ref		*/	IDC_RX_LV,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_SV,
		/* ref		*/	IDC_RX_LV,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{
		/* id		*/	IDC_RX_SV_ARROW,
		/* ref		*/	IDC_RX_LV_ARROW,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_RX_SV_ARROW,
		/* ref		*/	IDC_RX_LV_ARROW,
		/* flags	*/	ALIGN_T_B | ALIGN_H,
		/* value	*/	MARGIN
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_RX_SV);
	static_arrow_direction_set(dlg, IDC_RX_SV, -1);
	static_paint_function_set(dlg, IDC_RX_SV, _gui_paint);
	arrow_init(dlg, device, IDC_RX_SV_ARROW);
	arrow_type_set(dlg, IDC_RX_SV_ARROW, ARROW_LEFT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GUI window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control;
	U32		enable;
	U32		mode;
	U32		status;

	for (;;)	// A convenience loop.
	{
		if (device->handle == NULL)
		{
			// The device is closed.
			ctrl_menu_set(dlg, IDC_RX_SV, CTRL_DISABLE);
			ctrl_menu_set(dlg, IDC_RX_SV_ARROW, CTRL_DISABLE);
			break;
		}

		// The device is open.
		status	= HPDI32_CABLE_COMMAND_MODE__SV_GET(device->handle, &mode);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (mode != HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL)
		{
			// The signal is configured as GPIO.
			ctrl_menu_set(dlg, IDC_RX_SV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_SV_ARROW, CTRL_DISABLE);
			break;
		}

		// The signal is configured as Flow Control.
		status	= HPDI32_RX_ENABLE__GET(device->handle, &enable);
		error_status(dlg, IDC_INFO_TEXT, status);

		if (enable == HPDI32_RX_ENABLE_YES)
		{
			// The signal is input to the receiver.
			ctrl_menu_set(dlg, IDC_RX_SV, CTRL_ENABLE);
			ctrl_menu_set(dlg, IDC_RX_SV_ARROW, CTRL_ENABLE);
			break;
		}

		// The signal is unused by the receiver at this time.
		ctrl_menu_set(dlg, IDC_RX_SV, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_RX_SV_ARROW, CTRL_DISABLE);
		break;
	}

	control	= GetDlgItem(dlg, IDC_RX_SV);
	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	rx_status_valid_service
*
*	Purpose:
*
*		Implement the common service entry point for the Rx Status Valid
*		services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT rx_status_valid_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


