// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/tx_enabled.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement support for the Tx Enabled configuration option.
*
*****************************************************************************/

#include <windows.h>

#include "tx.h"
#include "align.h"
#include "arrow.h"
#include "resource.h"
#include "static.h"
#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	_gui_paint
*
*	Purpose:
*
*		Provide additional painting support for the GUI Tx Enabled control.
*
*	Arguments:
*
*		control		The handle to the control.
*
*		data		The control specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _gui_paint(HWND control, static_t* data)
{
	static const wave_t	down[]	=
	{
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_TOP,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	static const wave_t	up[]	=
	{
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	6
		},
		{
		/* flags	*/	WAVE_BOTTOM,
		/* skip		*/	0,
		/* count	*/	1,
		/* w1		*/	1,
		/* w2		*/	0
		},
		{ 0 }
	};

	int	x;
	int	y;

	x	= data->inside.rect.left
		+ data->arrow.offset
		+ data->arrow.width
		+ MARGIN;
	y	= data->inside.rect.top;

	x	+= static_text_paint(control, data, x, y, " Enabled ");
	x	+= static_wave_paint(data, x, y, up);

	x	+= static_text_paint(control, data, x, y, "   Disabled ");
	static_wave_paint(data, x, y, down);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		// Tx Enabled
		{
		/* id		*/	IDC_TX_ENABLED,
		/* ref		*/	IDC_TX_ENABLE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_ENABLED,
		/* ref		*/	IDC_TX_CLOCK,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_ENABLED,
		/* ref		*/	IDC_TX_ENABLE_ARROW,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	0
		},

		// Tx Enabled Arrow
		{
		/* id		*/	IDC_TX_ENABLED_ARROW,
		/* ref		*/	IDC_TX_ENABLE,
		/* flags	*/	ALIGN_T_T | ALIGN_B_B,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_ENABLED_ARROW,
		/* ref		*/	IDC_TX_CLOCK_ARROW,
		/* flags	*/	ALIGN_L_L | ALIGN_R_R,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_TX_ENABLED_ARROW,
		/* ref		*/	IDC_TX_ENABLED,
		/* flags	*/	ALIGN_L_R | ALIGN_W,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	static_init(dlg, device, IDC_TX_ENABLED);
	static_arrow_direction_set(dlg, IDC_TX_ENABLED, +1);
	static_paint_function_set(dlg, IDC_TX_ENABLED, _gui_paint);
	arrow_init(dlg, device, IDC_TX_ENABLED_ARROW);
	arrow_type_set(dlg, IDC_TX_ENABLED_ARROW, ARROW_RIGHT);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	HWND	control	= GetDlgItem(dlg, IDC_TX_ENABLED);
	U32		enable;
	UINT	flags;
	U32		status;
	U32		tri;

	if (device->handle)
	{
		ctrl_menu_set(dlg, IDC_TX_ENABLED, CTRL_ENABLE);
		ctrl_menu_set(dlg, IDC_TX_ENABLED_ARROW, CTRL_ENABLE);
		status	= HPDI32_TX_ENABLE__GET(device->handle, &enable);
		error_status(dlg, IDC_INFO_TEXT, status);
		status	= HPDI32_MISC_TX_RX_TRI_STATE__GET(device->handle, &tri);
		error_status(dlg, IDC_INFO_TEXT, status);

		if ((enable == HPDI32_TX_ENABLE_YES) ||
			(tri == HPDI32_MISC_TX_RX_TRI_STATE_DISABLE))
		{
			flags	= CTRL_ENABLE;
		}
		else
		{
			flags	= CTRL_DISABLE;
		}

		ctrl_menu_set(dlg, IDC_TX_ENABLED_ARROW, flags);
	}
	else
	{
		ctrl_menu_set(dlg, IDC_TX_ENABLED, CTRL_DISABLE);
		ctrl_menu_set(dlg, IDC_TX_ENABLED_ARROW, CTRL_DISABLE | CTRL_NULL);
	}

	InvalidateRect(control, NULL, TRUE);
	return(0);
}



/*****************************************************************************
*
*	Function:	tx_enabled_service
*
*	Purpose:
*
*		Implement the common service entry point for the Tx Enable services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT tx_enabled_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service			function			*/
		{ SERVICE_INIT,		_service_init		},
		{ SERVICE_REFRESH,	_service_refresh	},
		{ SERVICE_END_LIST,	NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


