// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/visual.h $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Export common items for the application.
*
*****************************************************************************/

#ifndef __VISUAL_H__
#define	__VISUAL_H__

#include "hpdi32.h"



/* #defines	*****************************************************************/

#define	VERSION_STRING		"1.5"
// 1.5  Removed compile warnings and errors in 64-bit GCC.
// 1.04	Fixed a bug in the GPIO data transfer code: the windows opened too small.
//		Ported to Microsoft Visual Studio 2005
//		Ported to 64-bit Windows.
//		Renamed hpdi32_api.* hpdi32.*.
// 1.03	Ported to the SDK.
/* 1.02	Corrected cable commands data for the Configuration Wizard (low+low -> low+high */
/* 1.01	Added I/O Single Cycle support. */
/* 1.00	Initial release. */

#define	APP_TITLE			"Visual HPDI32"

#define	VAR_STR_MACRO(m)	m, #m

#define	RGB_RED				RGB(255,0,0)

#define	ARRAY_ELEMENTS(a)	(sizeof((a))/sizeof((a)[0]))
#define	RECT_HEIGHT(r)		((r).bottom-(r).top)
#define	RECT_WIDTH(r)		((r).right-(r).left)

#define	MARGIN				2

#define	WAVE_BOTTOM			0x0001
#define	WAVE_TOP			0x0002
#define	WAVE_T_B			(WAVE_BOTTOM | WAVE_TOP)



/* data types	*************************************************************/

typedef struct
{
	int		app_buf;
	U32		buf_size;
} hpdi32_io_t;

typedef struct
{
	COLORREF	color;
	HBRUSH		brush;
	HPEN		pen;
} paint_object_t;

typedef struct
{
	U32			api_status;	// From hpdi32_api_status().
	int			usable;		// Is the HPDI32 API Library usable?
	U8			count;		// Count of installed boards.

	U8			index;		// Index of open board.
	void*		handle;		// handle of open device. NULL if not open.
	U32			oscillator;

	HINSTANCE	instance;
	int			margin;		// Used by the GUI.
	int			btn_height;

	hpdi32_io_t	tx;
	hpdi32_io_t	rx;

	struct
	{
		HCURSOR		arrows;
		HCURSOR		cross;
		HCURSOR		cross_x;
		HCURSOR		cross_y;
		HCURSOR		wait;
	} cursor;

	struct
	{
		HFONT	defalt;		// 8 point
		HFONT	defalt_b;	// 8 point, bold
		HFONT	defalt_bl;	// 10 point, bold
		HFONT	defalt_bll;	// 12 point bold
		HFONT	fixed;		// 8 point
		HFONT	fixed_l;	// 10 point
	} font;

	struct
	{
		HICON	big;
		HICON	little;
	} icon;

	struct
	{
		paint_object_t	background;
		paint_object_t	black;
		paint_object_t	btnface;
		paint_object_t	btnhighlt;
		paint_object_t	btnshadow;
		paint_object_t	btntext;
		paint_object_t	btntext_dot;
		paint_object_t	gray_dk;
		paint_object_t	gray_lt;
		paint_object_t	green;
		paint_object_t	red;
		paint_object_t	white;
		paint_object_t	window;
		paint_object_t	window_text;
		paint_object_t	x_cursor;
		paint_object_t	y_cursor;
	} paint;

} device_t;

typedef struct
{
	const char*	user_text;
	U32			value;
	const char*	value_text;
	const char*	suffix;
} value_list_entry_t;

typedef struct
{
	const value_list_entry_t*	list;
	size_t						size;
} value_list_t;

typedef struct
{
	U32	min;
	U32	max;
	int	k;		// This should be either 1024  or 1000.
} value_range_t;

typedef enum
{
	VALUE_TYPE_LIST,
	VALUE_TYPE_RANGE
} value_type_t;

typedef struct
{
	value_type_t	type;

	union
	{
		const void*				vp;
		const value_list_t*		list;
		const value_range_t*	range;
	} u;
} value_t;

typedef struct _parm_entry_t parm_entry_t;

struct _parm_entry_t
{
	const char*		title;		// Human readable text.
	U32				parm;
	const char*		parm_text;	// NULL terminates list.
	const char*		suffix;		// Appears between "XXX__" and "SET".
	U32				which;
	const char*		set_arg;	// Appears between "set" value and ")".
	const char*		get_arg;	// Appears between "set" value and ")".
	const value_t*	values;
	U32				(*cfg_wiz)(	HWND				dlg,
								device_t*			device,
								const parm_entry_t*	entry,
								U32					value);
	const char*		desc;
};

typedef struct
{
	const parm_entry_t**	list;
	size_t					size;
	const char*				suffix;	// For override purposes.
	U32						which;	// For override purposes.
} parm_list_t;

typedef struct
{
	ULONG	flags;
	int		skip;
	int		count;
	int		w1;
	int		w2;
} wave_t;



/* prototypes	*************************************************************/

void	error_status(HWND dlg, int id, U32 status);
void	format_float(float value, char* dest);
void	format_u32(U32 value, int k, char* dest);
U32		hex_text_2_u32(const char* src, U32* u32);
void	main_close_all(HWND dlg, device_t* device);
void	main_refresh(device_t* device, const char* msg);
void	main_refresh_add(HWND dlg, void (*func)(HWND dlg, device_t* device, const char* msg));
U32		numeric_text_2_u32(const char* src, int k, U32* u32);
void	win_pos_restore(HWND win, const char* name, int sizable);
void	win_pos_save(HWND win, const char* name);

void	set_win_title(	HWND			win,
						const device_t*	device,
						int				version,
						int				index,
						const char*		suffix);



#endif
