// HPDI32 - Application - Any OS - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/xfer/config.c $
// $Rev: 31049 $
// $Date: 2014-10-27 17:49:58 -0500 (Mon, 27 Oct 2014) $

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "hpdi32.h"
#include "main.h"
#include "hpdi32_util.h"



/* #defines	****************************************************************/

#define	VERBOSE_OFF		0
#define	VERBOSE_ON		1
#define	VERBOSE_VERY	2



/* data types	************************************************************/

typedef struct
{
	U32*	var;
} var_hex_t;

typedef struct
{
	const char*	text;
	U32			value;
} var_item_t;

typedef struct
{
	const var_item_t*	list;
	size_t				size;
	U32*				var;
} var_list_t;

typedef struct
{
	U32	which;
	U32	parm;
} parm_hex_t;

typedef struct
{
	const char*	text;
	U32			which;
	U32			parm;
	U32			value;
} parm_item_t;

typedef struct
{
	const parm_item_t*	list;
	size_t				size;
} parm_list_t;

typedef struct _cmd_t
{
	const char*	command;
	void*		data;
	U32			(*func)(void*					device,
						long					line,
						const char*				value,
						const struct _cmd_t*	cmd,
						U32*					set,
						unsigned long*			get);
} cmd_t;



/* prototypes	*************************************************************/

static U32	_list_commands(void* device, long line, const char* value, const cmd_t* cmd, U32* set, unsigned long* get);
static U32	_parm_hex(void* device, long line, const char* value, const cmd_t* cmd, U32* set, unsigned long* get);
static U32	_parm_list(void* device, long line, const char* value, const cmd_t* cmd, U32* set, unsigned long* get);
static U32	_var_hex(void* device, long line, const char* value, const cmd_t* cmd, U32* set, unsigned long* get);
static U32	_var_list(void* device, long line, const char* value, const cmd_t* cmd, U32* set, unsigned long* get);



/* variables	*************************************************************/

static	U32	_list		= 0;
static	U32	_verbose	= 0;



static const parm_hex_t	_almost_empty_hex	=
{
	/* which	*/	HPDI32_WHICH_TX_RX | HPDI32_WHICH_AE,
	/* parm		*/	HPDI32_FIFO_ALMOST_LEVEL
};



static const parm_hex_t	_almost_full_hex	=
{
	/* which	*/	HPDI32_WHICH_TX_RX | HPDI32_WHICH_AF,
	/* parm		*/	HPDI32_FIFO_ALMOST_LEVEL
};



static const var_hex_t	_block_count_hex	=
{
	/* var	*/	&block_count
};



static const var_hex_t	_block_samples_hex	=
{
	/* var	*/	&block_samples
};



static const parm_item_t	_data_size_items[]	=
{
	{
		/* text		*/	"8",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_DATA_SIZE,
		/* value	*/	HPDI32_IO_DATA_SIZE_8_BITS
	},
	{
		/* text		*/	"16",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_DATA_SIZE,
		/* value	*/	HPDI32_IO_DATA_SIZE_16_BITS
	},
	{
		/* text		*/	"32",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_DATA_SIZE,
		/* value	*/	HPDI32_IO_DATA_SIZE_32_BITS
	}
};

static const parm_list_t	_data_size_list	=
{
	/* list	*/	_data_size_items,
	/* size	*/	ARRAY_ELEMENTS(_data_size_items)
};



static const var_hex_t	_io_clock_hex	=
{
	/* var	*/	&io_clock
};



static const parm_item_t	_mode_items[]	=
{
	{
		/* text		*/	"PIO",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_MODE,
		/* value	*/	HPDI32_IO_MODE_PIO
	},
	{
		/* text		*/	"DMA",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_MODE,
		/* value	*/	HPDI32_IO_MODE_DMA
	},
	{
		/* text		*/	"DMDMA",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_MODE,
		/* value	*/	HPDI32_IO_MODE_DMDMA
	}
};

static const parm_list_t	_mode_list	=
{
	/* list	*/	_mode_items,
	/* size	*/	ARRAY_ELEMENTS(_mode_items)
};



static const parm_item_t	_status_valid_mirror_items[]	=
{
	{
		/* text		*/	"No",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_TX_STATUS_VALID_MIRROR,
		/* value	*/	HPDI32_TX_STATUS_VALID_MIRROR_DISABLE
	},
	{
		/* text		*/	"Yes",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_TX_STATUS_VALID_MIRROR,
		/* value	*/	HPDI32_TX_STATUS_VALID_MIRROR_ENABLE
	}
};

static const parm_list_t	_status_valid_mirror_list	=
{
	/* list	*/	_status_valid_mirror_items,
	/* size	*/	ARRAY_ELEMENTS(_status_valid_mirror_items)
};



static const parm_item_t	_frame_valid_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_FRAME_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_FRAME_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_FRAME_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_FRAME_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_frame_valid_list	=
{
	/* list	*/	_frame_valid_items,
	/* size	*/	ARRAY_ELEMENTS(_frame_valid_items)
};



static const parm_item_t	_line_valid_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_LINE_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_LINE_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_LINE_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_LINE_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_line_valid_list	=
{
	/* list	*/	_line_valid_items,
	/* size	*/	ARRAY_ELEMENTS(_line_valid_items)
};



static const parm_hex_t	_line_valid_off_count_hex	=
{
	/* which	*/	0,
	/* parm		*/	HPDI32_TX_LINE_VALID_OFF_COUNT
};



static const parm_hex_t	_line_valid_on_count_hex	=
{
	/* which	*/	0,
	/* parm		*/	HPDI32_TX_LINE_VALID_ON_COUNT
};



static const var_item_t	_list_items[]	=
{
	{
		/* text		*/	"No",
		/* value	*/	0
	},
	{
		/* text		*/	"Yes",
		/* value	*/	1
	}
};

static const var_list_t	_list_list	=
{
	/* list	*/	_list_items,
	/* size	*/	ARRAY_ELEMENTS(_list_items),
	/* var	*/	&_list
};



static const parm_item_t	_single_cycle_items[]	=
{
	{
		/* text		*/	"Absent",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_SINGLE_CYCLE,
		/* value	*/	HPDI32_IO_SINGLE_CYCLE_ABSENT
	},
	{
		/* text		*/	"Present",
		/* which	*/	HPDI32_WHICH_TX_RX,
		/* parm		*/	HPDI32_IO_SINGLE_CYCLE,
		/* value	*/	HPDI32_IO_SINGLE_CYCLE_PRESENT
	}
};

static const parm_list_t	_single_cycle_list	=
{
	/* list	*/	_single_cycle_items,
	/* size	*/	ARRAY_ELEMENTS(_single_cycle_items)
};



static const parm_item_t	_status_valid_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_STATUS_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_STATUS_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_STATUS_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_STATUS_VALID_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_status_valid_list	=
{
	/* list	*/	_status_valid_items,
	/* size	*/	ARRAY_ELEMENTS(_status_valid_items)
};



static const parm_hex_t	_status_valid_count_hex	=
{
	/* which	*/	0,
	/* parm		*/	HPDI32_TX_STATUS_VALID_COUNT
};



static const parm_hex_t	_timeout_hex	=
{
	/* which	*/	HPDI32_WHICH_TX_RX,
	/* parm		*/	HPDI32_IO_TIMEOUT
};



static const parm_item_t	_rx_ready_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_RX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_RX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_RX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_RX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};


static const parm_list_t	_rx_ready_list	=
{
	/* list	*/	_rx_ready_items,
	/* size	*/	ARRAY_ELEMENTS(_rx_ready_items)
};



static const parm_hex_t	_tx_clock_divider_hex	=
{
	/* which	*/	0,
	/* parm		*/	HPDI32_TX_CLOCK_DIVIDER
};



static const parm_item_t	_tx_ready_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_TX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_TX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_TX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_TX_READY_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_tx_ready_list	=
{
	/* list	*/	_tx_ready_items,
	/* size	*/	ARRAY_ELEMENTS(_tx_ready_items)
};



static const parm_item_t	_tx_enabled_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_TX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_TX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_TX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_TX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_tx_enabled_list	=
{
	/* list	*/	_tx_enabled_items,
	/* size	*/	ARRAY_ELEMENTS(_tx_enabled_items)
};



static const parm_item_t	_rx_enabled_items[]	=
{
	{
		/* text		*/	"Low",
		/* which	*/	HPDI32_WHICH_RX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW
	},
	{
		/* text		*/	"High",
		/* which	*/	HPDI32_WHICH_RX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI
	},
	{
		/* text		*/	"In",
		/* which	*/	HPDI32_WHICH_RX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_GPIO_IN
	},
	{
		/* text		*/	"FlowControl",
		/* which	*/	HPDI32_WHICH_RX_ENABLED_,
		/* parm		*/	HPDI32_CABLE_COMMAND_MODE,
		/* value	*/	HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL
	}
};

static const parm_list_t	_rx_enabled_list	=
{
	/* list	*/	_rx_enabled_items,
	/* size	*/	ARRAY_ELEMENTS(_rx_enabled_items)
};



static const var_item_t	_verbose_items[]	=
{
	{
		/* text		*/	"No",
		/* value	*/	VERBOSE_OFF
	},
	{
		/* text		*/	"Yes",
		/* value	*/	VERBOSE_ON
	},
	{
		/* text		*/	"Very",
		/* value	*/	VERBOSE_VERY
	}
};

static const var_list_t	_verbose_list	=
{
	/* list	*/	_verbose_items,
	/* size	*/	ARRAY_ELEMENTS(_verbose_items),
	/* var	*/	&_verbose
};



static const cmd_t	_commands[]	=
{
	{
		/* command	*/	"AlmostEmpty",
		/* arg		*/	(void*) &_almost_empty_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"AlmostFull",
		/* arg		*/	(void*) &_almost_full_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"BlockCount",
		/* arg		*/	(void*) &_block_count_hex,
		/* func		*/	_var_hex
	},
	{
		/* command	*/	"BlockSamples",
		/* arg		*/	(void*) &_block_samples_hex,
		/* func		*/	_var_hex
	},
	{
		/* command	*/	"DataSize",
		/* arg		*/	(void*) &_data_size_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"IOClock",
		/* arg		*/	(void*) &_io_clock_hex,
		/* func		*/	_var_hex
	},
	{
		/* command	*/	"FrameValid",
		/* arg		*/	(void*) &_frame_valid_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"LineValid",
		/* arg		*/	(void*) &_line_valid_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"LineValidOffCount",
		/* arg		*/	(void*) &_line_valid_off_count_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"LineValidOnCount",
		/* arg		*/	(void*) &_line_valid_on_count_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"List",
		/* arg		*/	(void*) &_list_list,
		/* func		*/	_list_commands
	},
	{
		/* command	*/	"Mode",
		/* arg		*/	(void*) &_mode_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"RxEnabled",
		/* arg		*/	(void*) &_rx_enabled_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"RxReady",
		/* arg		*/	(void*) &_rx_ready_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"SingleCycle",
		/* arg		*/	(void*) &_single_cycle_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"StatusValid",
		/* arg		*/	(void*) &_status_valid_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"StatusValidCount",
		/* arg		*/	(void*) &_status_valid_count_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"StatusValidMirror",
		/* arg		*/	(void*) &_status_valid_mirror_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"Timeout",
		/* arg		*/	(void*) &_timeout_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"TxClockDivider",
		/* arg		*/	(void*) &_tx_clock_divider_hex,
		/* func		*/	_parm_hex
	},
	{
		/* command	*/	"TxEnabled",
		/* arg		*/	(void*) &_tx_enabled_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"TxReady",
		/* arg		*/	(void*) &_tx_ready_list,
		/* func		*/	_parm_list
	},
	{
		/* command	*/	"Verbose",
		/* arg		*/	(void*) &_verbose_list,
		/* func		*/	_var_list
	}
};



/*****************************************************************************
*
*	Function:	_parse_u32
*
*	Purpose:
*
*		Parse a value and return it as a U32.
*
*	Arguments:
*
*		line		The line number of the file being processed.
*
*		command		The command being processed.
*
*		value		The text of the value to process.
*
*		get			The value parsed goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _parse_u32(long line, const char* command, const char* value, U32* get)
{
	char			c;
	int				i;
	U32				status	= GSC_SUCCESS;
	unsigned long	ul;


	if ((value[0] == '0') && (value[1] == 'x'))
		i	= sscanf(value, "0x%lx%c", &ul, &c);
	else if (value[0] == '-')
		i	= sscanf(value, "%ld%c", &ul, &c);
	else
		i	= sscanf(value, "%lu%c", &ul, &c);

	if (i == 1)
	{
		get[0]	= (U32) ul;
	}
	else
	{
		printf(	"ERROR: invalid value on line %ld: '%s=%s'\n",
				line,
				command,
				value);
		status	= GSC_FAILED;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_parm_hex
*
*	Purpose:
*
*		Process a command whose VALUE is a hexadecimal parameter values.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		line		The line number being processed.
*
*		value		The text of the value to process.
*
*		cmd			The command structure for the command being processed.
*
*		set			The value applied goes here.
*
*		get			The value achieved goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _parm_hex(	void*			device,
						long			line,
						const char*		value,
						const cmd_t*	cmd,
						U32*			set,
						unsigned long*	get)
{
	const parm_hex_t*	hex		= (const parm_hex_t*) cmd->data;
	U32					status	= GSC_SUCCESS;
	U32					u32;

	status	= _parse_u32(line, cmd->command, value, &u32);

	if (status == GSC_SUCCESS)
	{
		set[0]	= u32;
		status	= hpdi32_config(device, hex->parm, hex->which, u32, get);

		if (status != GSC_SUCCESS)
		{
			printf(	"ERROR: operation failed on line %ld: 0x%lX: '%s=%s'\n",
					line,
					(long) status,
					cmd->command,
					value);
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_parm_list
*
*	Purpose:
*
*		Process a command whose VALUE corelates to a list of predefined
*		parameter values.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		line		The line number being processed.
*
*		value		The text of the value to process.
*
*		cmd			The command structure for the command being processed.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _parm_list(	void*			device,
						long			line,
						const char*		value,
						const cmd_t*	cmd,
						U32*			set,
						unsigned long*	get)
{
	int					i;
	const parm_list_t*	list	= (const parm_list_t*) cmd->data;
	U32					status	= GSC_SUCCESS;

	for (i = 0; i < (int) list->size; i++)
	{
		if (stricmp(value, list->list[i].text) == 0)
			break;
	}

	if (i < (int) list->size)
	{
		set[0]	= list->list[i].value;
		status	= hpdi32_config(device,
								list->list[i].parm,
								list->list[i].which,
								list->list[i].value,
								get);

		if (status != GSC_SUCCESS)
		{
			printf(	"ERROR: operation failed on line %ld: 0x%lX: '%s=%s'\n",
					line,
					(long) status,
					cmd->command,
					value);
		}
	}
	else
	{
		printf(	"ERROR: invalid value on line %ld: '%s=%s'\n",
				line,
				cmd->command,
				value);
		status	= GSC_FAILED;
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_var_hex
*
*	Purpose:
*
*		Process a command whose VALUE is a hexadecimal variable values.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		line		The line number being processed.
*
*		value		The text of the value to process.
*
*		cmd			The command structure for the command being processed.
*
*		set			The value applied goes here.
*
*		get			The value achieved goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _var_hex(	void*			device,
						long			line,
						const char*		value,
						const cmd_t*	cmd,
						U32*			set,
						unsigned long*	get)
{
	const var_hex_t*	hex		= (const var_hex_t*) cmd->data;
	U32					status	= GSC_SUCCESS;
	U32					u32;

	status	= _parse_u32(line, cmd->command, value, &u32);

	if (status == GSC_SUCCESS)
	{
		set[0]		= u32;
		hex->var[0]	= u32;
		get[0]		= hex->var[0];
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_var_list
*
*	Purpose:
*
*		Process a command whose VALUE corelates to a list of predefined
*		values.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		line		The line number being processed.
*
*		value		The text of the value to process.
*
*		cmd			The command structure for the command being processed.
*
*		set			The value applied goes here.
*
*		get			The value achieved goes here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _var_list(	void*			device,
						long			line,
						const char*		value,
						const cmd_t*	cmd,
						U32*			set,
						unsigned long*	get)
{
	int					i;
	const var_list_t*	list	= (const var_list_t*) cmd->data;
	U32					status;

	for (i = 0; i < (int) list->size; i++)
	{
		if (stricmp(value, list->list[i].text) == 0)
			break;
	}

	if (i < (int) list->size)
	{
		status			= GSC_SUCCESS;
		set[0]			= list->list[i].value;
		list->var[0]	= list->list[i].value;
		get[0]			= list->list[i].value;
	}
	else
	{
		printf(	"ERROR: invalid value on line %ld: '%s=%s'\n",
				line,
				cmd->command,
				value);
		status	= GSC_FAILED;
	}

	return(status);
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_list_commands
*
*	Purpose:
*
*		Process a command whose VALUE corelates to a list of predefined
*		values.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		line		The line number being processed.
*
*		value		The text of the value to process.
*
*		cmd			The command structure for the command being processed.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _list_commands(	void*			device,
							long			line,
							const char*		value,
							const cmd_t*	cmd,
							U32*			set,
							unsigned long*	get)
{
	int					i;
	int					j;
	int					len	= 1;
	const parm_hex_t*	parm_hex;
	const parm_list_t*	parm_list;
	const char*			ptr;
	U32					status;
	unsigned long		ul;
	const var_hex_t*	var_hex;
	const var_list_t*	var_list;

	status	= _var_list(device, line, value, cmd, set, get);

	if ((status == GSC_SUCCESS) && (_list))
	{
		printf("Commands\n");

		for (i = 0; i < (int) ARRAY_ELEMENTS(_commands); i++)
		{
			j	= (int) strlen(_commands[i].command);

			if (len < j)
				len	= j;
		}

		for (i = 0; i < (int) ARRAY_ELEMENTS(_commands); i++)
		{
			printf("%5d. %-*s = ", i + 1, len, _commands[i].command);

			if ((_commands[i].func == _list_commands) ||
				(_commands[i].func == _var_list))
			{
				var_list	= (const var_list_t*) _commands[i].data;
				ptr			= NULL;

				for (j = 0; j < (int) var_list->size; j++)
				{
					if (var_list->var[0] == var_list->list[j].value)
					{
						ptr	= var_list->list[j].text;
						break;
					}
				}

				if (ptr)
					printf("%s (", ptr);

				for (j = 0; j < (int) var_list->size; j++)
				{
					if (j)
						printf(", ");

					printf("%s", var_list->list[j].text);
				}

				if (ptr)
					printf(")");

				printf("\n");
			}
			else if (_commands[i].func == _parm_hex)
			{

				parm_hex	= (const parm_hex_t*) _commands[i].data;
				status		= hpdi32_config(device,
											parm_hex->parm,
											parm_hex->which,
											GSC_NO_CHANGE,
											&ul);

				if (status != GSC_SUCCESS)
				{
					printf("ERROR: retrieval failed\n");
				}
				else
				{
					format_value(ul, "");
					printf(" (hex or decimal number)\n");
				}
			}
			else if (_commands[i].func == _parm_list)
			{
				parm_list	= (const parm_list_t*) _commands[i].data;
				ptr			= NULL;

				for (j = 0; j < (int) parm_list->size; j++)
				{
					status	= hpdi32_config(device,
											parm_list->list[j].parm,
											parm_list->list[j].which,
											GSC_NO_CHANGE,
											&ul);

					if (status != GSC_SUCCESS)
						break;

					if (ul == parm_list->list[j].value)
					{
						ptr	= parm_list->list[j].text;
						break;
					}
				}

				if (ptr)
					printf("%s (", ptr);

				for (j = 0; j < (int) parm_list->size; j++)
				{
					if (j)
						printf(", ");

					printf("%s", parm_list->list[j].text);
				}

				if (ptr)
					printf(")");

				printf("\n");
			}
			else if (_commands[i].func == _var_hex)
			{
				var_hex	= (const var_hex_t*) _commands[i].data;
				ul		= var_hex->var[0];
				format_value(ul, "");
				printf(" (hex or decimal number)\n");
			}
			else
			{
				printf("INCOMPLETE\n");
			}
		}

		printf("\n");
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_read_line
*
*	Purpose:
*
*		Read the configuration file.
*
*	Arguments:
*
*		file		The file pointer to access.
*
*		buf			The receive buffer.
*
*		size		The size of the above buffer.
*
*		end			Did we hit the end of the file?
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

static U32 _read_line(FILE* file, char* buf, size_t size, int* end)
{
	int		eof;
	size_t	got;
	int		i;
	long	pos;
	U32		status;

	end[0]	= 0;
	pos		= ftell(file);
	got		= fread(buf, 1, size, file);
	eof		= feof(file);

	if (got == 0)
	{
		if (eof)
		{
			// We're at the end of the file.
			end[0]	= 1;
			status	= GSC_SUCCESS;
		}
		else
		{
			printf("ERROR: file read error, errno = %d\n", errno);
			status	= GSC_FAILED;
		}
	}
	else
	{
		// Locate the end of the line within the buffer adjust the file
		// pointer accordingly.

		status	= GSC_SUCCESS;

		for (i = 0;; i++)
		{
			if (i == (int) got)
			{
				buf[i]	= 0;
				break;
			}

			if (buf[i] == '\n')
			{
				// UNIX line ending.
				buf[i]	= 0;
				fseek(file, pos + i + 1, SEEK_SET);
				break;
			}

			if (buf[i] == '\r')
			{
				// A DOS line ending.

				buf[i]	= 0;

				if (((i + 1) < (int) got) && (buf[i + 1] == '\n'))
					i++;

				fseek(file, pos + i + 1, SEEK_SET);
				break;
			}
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	_skip_ws
*
*	Purpose:
*
*		Skip the white space at the beginning of the given buffer.
*
*	Arguments:
*
*		buf		The source to process.
*
*	Returned:
*
*		char*	The modified buffer.
*
*****************************************************************************/

static char* _skip_ws(char* buf)
{
	for (;;)
	{
		if (buf[0] == 0)
			break;

		if (isspace(buf[0]))
			buf++;
		else
			break;
	}

	return(buf);
}



/*****************************************************************************
*
*	Function:	_trunk_ws
*
*	Purpose:
*
*		Remove the trailing white space from the given buffer.
*
*	Arguments:
*
*		buf		The source to process.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _trunk_ws(char* buf)
{
	char*	end	= buf + strlen(buf) - 1;

	for (;;)
	{
		if (buf[0] == 0)
			break;

		if (isspace(end[0]))
		{
			end[0]	= 0;
			end--;
		}
		else
		{
			break;
		}
	}
}



/*****************************************************************************
*
*	Function:	_parse_key_value
*
*	Purpose:
*
*		Parse a line into a KEY=VALUE component pair.
*
*	Arguments:
*
*		buf		The source to parse.
*
*		key		The KEY pointer is recorded here.
*
*		eq		This points to where the equal sign was.
*
*		value	The VALUE pointer is recorded here.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _parse_key_value(char* buf, char** key, char** eq, char** value)
{
	char*	semi;

	buf	= _skip_ws(buf);
	semi	= strchr(buf, ';');

	if (semi)
		semi[0]	= 0;	// Truncate at the comment.

	eq[0]	= strchr(buf, '=');

	if (buf[0] == 0)
	{
		// An empty line.
		buf[0]		= 0;
		key[0]		= buf;
		value[0]	= buf;
	}
	else if (eq[0])
	{
		// Form: KEY=VALUE
		eq[0][0]	= 0;
		key[0]		= buf;
		value[0]	= _skip_ws(eq[0] + 1);
		_trunk_ws(buf);
		_trunk_ws(value[0]);
	}
	else
	{
		// Form: KEY
		key[0]		= buf;
		value[0]	= buf + strlen(buf);
		_trunk_ws(buf);
	}
}



/*****************************************************************************
*
*	Function:	_command_locate
*
*	Purpose:
*
*		Locate the command structure for the given command.
*
*	Arguments:
*
*		cmd		The command to locate.
*
*	Returned:
*
*		NULL	The command wasn't found.
*		else	The structure for the given command.
*
*****************************************************************************/

static const cmd_t* _command_locate(const char* cmd)
{
	const cmd_t*	ptr	= NULL;
	int				i;

	for (i = 0; i < (int) ARRAY_ELEMENTS(_commands); i++)
	{
		if (stricmp(cmd, _commands[i].command) == 0)
		{
			ptr	= &_commands[i];
			break;
		}
	}

	return(ptr);
}



/*****************************************************************************
*
*	Function:	read_config_file
*
*	Purpose:
*
*		Read the configuration file.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*		name		The name of the file to access.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 read_config_file(void* device, const char* name)
{
	char			buf[1024];
	const cmd_t*	cmd;
	int				end;
	char*			eq;
	unsigned long	get;
	FILE*			file;
	char*			key;
	long			line	= 0;
	U32				set;
	U32				status;
	char			tmp[1024];
	char*			value;

	file	= fopen(name, "rb");
	status	= file ? GSC_SUCCESS : GSC_FAILED;

	for (; file;)
	{
		line++;
		status	= _read_line(file, buf, sizeof(buf), &end);

		if ((status != GSC_SUCCESS) || (end))
			break;

		memcpy(tmp, buf, sizeof(tmp));
		_parse_key_value(buf, &key, &eq, &value);

		if ((key[0] == 0) && (eq == NULL) && (value[0] == 0))
			continue;

		if ((key[0] == 0) || (eq == NULL) || (value[0] == 0))
		{
			printf("ERROR: invalid syntax on line %ld: '%s'\n", line, tmp);
			status	= GSC_FAILED;
			break;
		}

		cmd	= _command_locate(key);

		if (cmd == NULL)
		{
			printf("ERROR: unknown key on line %ld: '%s'\n", line, tmp);
			status	= GSC_FAILED;
			break;
		}

		status	= (cmd->func)(device, line, value, cmd, &set, &get);

		if (status != GSC_SUCCESS)
			break;

		if (_verbose)
		{
			printf("%5ld: %s=%s", line, key, value);

			if (_verbose == VERBOSE_VERY)
				printf(" (%lu -> %lu)", (long) set, (long) get);

			printf("\n");
		}

		continue;
	}

	fclose(file);
	return(status);
}


